@@ -26,6 +26,13 @@
 #include "logging_streambufs.h"
 #include "ri.h"
 #include "version.h"
+#include "bdec.h"
+#include "parserstate.h"
+
+#ifdef AQSIS_SYSTEM_WIN32
+#include <io.h>
+#endif // AQSIS_SYSTEM_WIN32
+#include <fcntl.h>
 
 #include <iostream>
 #include <iomanip>
@@ -95,23 +102,23 @@ std::string g_plugin_path;
 void RenderFile( FILE* file, std::string& name );
 
 // Command-line arguments
-bool g_cl_pause;
+ArgParse::apflag g_cl_pause;
 // Set verbose stats if in debug mode
 #ifdef	_DEBUG
-int g_cl_endofframe = 3;
+ArgParse::apint g_cl_endofframe = 3;
 #else
-int g_cl_endofframe = -1;
+ArgParse::apint g_cl_endofframe = -1;
 #endif
-bool g_cl_nostandard = 0;
-bool g_cl_help = 0;
-bool g_cl_version = 0;
-bool g_cl_fb = 0;
-bool g_cl_progress = 0;
-bool g_cl_Progress = 0;
-bool g_cl_rinfo = 0;
-bool g_cl_no_color = 0;
-bool g_cl_beep = 0;
-int g_cl_verbose = 1;
+ArgParse::apflag g_cl_nostandard = 0;
+ArgParse::apflag g_cl_help = 0;
+ArgParse::apflag g_cl_version = 0;
+ArgParse::apflag g_cl_fb = 0;
+ArgParse::apflag g_cl_progress = 0;
+ArgParse::apflag g_cl_Progress = 0;
+ArgParse::apflag g_cl_rinfo = 0;
+ArgParse::apflag g_cl_no_color = 0;
+ArgParse::apflag g_cl_beep = 0;
+ArgParse::apint g_cl_verbose = 1;
 ArgParse::apfloatvec g_cl_cropWindow;
 ArgParse::apstring g_cl_rc_path = "";
 ArgParse::apstring g_cl_shader_path = "";
@@ -127,9 +134,11 @@ ArgParse::apstring g_cl_mode = "rgba";
 ArgParse::apstring g_cl_strprogress = "Frame (%f) %p%% complete [ %s secs / %S left ]";
 ArgParse::apstring g_cl_framesList = "";
 ArgParse::apintvec g_cl_frames;
+ArgParse::apintvec g_cl_res;
+ArgParse::apstringvec g_cl_options;
 
 #ifdef	AQSIS_SYSTEM_POSIX
-bool g_cl_syslog = 0;
+ArgParse::apflag g_cl_syslog = 0;
 #endif	// AQSIS_SYSTEM_POSIX
 
 /// Function to print the progress of the render.  Used as the callback function to a RiProgressHandler call.
@@ -275,7 +284,7 @@ RtVoid PrintProgress( RtFloat percent, RtInt FrameNo )
 }
 
 
-/** Function to setup specific options needed after world loading but before rendering.
+/** Function to setup specific options needed after options are complete but before the world is created.
 	Used as the callback function to a RiPreWorldFunction call.
  */
 #ifdef	AQSIS_SYSTEM_BEOS
@@ -308,7 +317,62 @@ RtVoid PreWorld()
     {
         RiCropWindow(g_cl_cropWindow[0], g_cl_cropWindow[1], g_cl_cropWindow[2], g_cl_cropWindow[3]);
     }
-    return ;
+
+   	// Pass in specified resolution.
+	if(g_cl_res.size() == 2)
+	{
+		RiFormat(g_cl_res[0], g_cl_res[1], 1.0f);
+	}
+
+	for( ArgParse::apstringvec::iterator i = g_cl_options.begin(); i != g_cl_options.end(); ++i )
+	{
+		// This is not pretty, gzopen attempts to read a gzip header
+		//from the stream, if we try and do this before we have sent
+		//the request the we get deadlock, so we have to create the decoder
+		//after the request, we use a buffer size of one to prevent any
+		//potential blocking.
+		librib::CqRibBinaryDecoder *decoder;
+
+		int hpipe[2];
+		FILE *readPipe, *writePipe;
+
+#ifdef AQSIS_SYSTEM_WIN32
+		if( _pipe( hpipe, i->length(), _O_TEXT) != -1 ) 
+#else
+		if( pipe( hpipe) != -1 ) 
+#endif
+		{
+			// Feed the string contents into the pipe so that the parser can read them out the other end.
+#ifdef AQSIS_SYSTEM_WIN32
+			readPipe = _fdopen(hpipe[0],"r");
+			writePipe = _fdopen(hpipe[1],"w");
+#else
+			readPipe = fdopen(hpipe[0],"r");
+			writePipe = fdopen(hpipe[1],"w");
+#endif
+
+			fwrite(i->c_str(),1,i->length(),writePipe);
+			fflush(writePipe);
+			fclose(writePipe);
+
+			decoder = new librib::CqRibBinaryDecoder( readPipe, 16384);
+
+			// Parse the resulting block of RIB.
+			librib::CqRIBParserState currstate = librib::GetParserState();
+
+			if( currstate.m_pParseCallbackInterface == NULL ) currstate.m_pParseCallbackInterface = new librib2ri::Engine;
+
+			librib::ParseOpenStream( decoder, "aqsis command line option", *(currstate.m_pParseCallbackInterface), *(currstate.m_pParseErrorStream), (RtArchiveCallback) RI_NULL );
+
+			librib::SetParserState( currstate );
+
+			delete( decoder );
+
+			fclose(readPipe);
+		}
+	}
+
+	return ;
 }
 
 
@@ -344,6 +408,7 @@ int main( int argc, const char** argv )
 	g_dso_path.append( "dsos" );
 	g_procedural_path.append( "procedures" );
 	g_plugin_path.append( "plugins" );
+	g_rc_path.append( "aqsisrc" );
 #elif AQSIS_SYSTEM_MACOSX
     CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
     CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle);
@@ -402,6 +467,16 @@ int main( int argc, const char** argv )
         ap.argFlag( "fb", "\aSame as --type=\"framebuffer\" --mode=\"rgb\"", &g_cl_fb );
         ap.alias( "fb", "d" );
         ap.argFloats( "crop", " x1 x2 y1 y2\aSpecify a crop window, values are in screen space.", &g_cl_cropWindow, ArgParse::SEP_ARGV, 4);
+		ap.argInts( "frames", " f1 f2\aSpecify a starting/ending frame to render (inclusive).", &g_cl_frames, ArgParse::SEP_ARGV, 2);
+		ap.argString( "frameslist", "=string\aSpecify a range of frames to render, ',' separated with '-' to indicate ranges.", &g_cl_framesList);
+        ap.argFlag( "nocolor", "\aDisable colored output", &g_cl_no_color );
+        ap.argFlag( "beep", "\aBeep on completion of all ribs", &g_cl_beep );
+        ap.alias( "nocolor", "nc" );
+		ap.argInts( "res", " x y\aSpecify the resolution of the render.", &g_cl_res, ArgParse::SEP_ARGV, 2);
+		ap.argStrings( "option", "=string\aA valid RIB Option string, can be specified multiple times.", &g_cl_options);
+#ifdef	AQSIS_SYSTEM_POSIX
+        ap.argFlag( "syslog", "\aLog messages to syslog", &g_cl_syslog );
+#endif	// AQSIS_SYSTEM_POSIX
         ap.argString( "rc", "=string\aOverride the default RIB configuration file [" + g_rc_path + "]", &g_cl_rc_path );
         ap.argString( "shaders", "=string\aOverride the default shader searchpath(s) [" + g_shader_path + "]", &g_cl_shader_path );
         ap.argString( "archives", "=string\aOverride the default archive searchpath(s) [" + g_archive_path + "]", &g_cl_archive_path );
@@ -410,14 +485,6 @@ int main( int argc, const char** argv )
         ap.argString( "dsolibs", "=string\aOverride the default dso searchpath(s) [" + g_dso_path + "]", &g_cl_dso_path );
         ap.argString( "procedurals", "=string\aOverride the default procedural searchpath(s) [" + g_procedural_path + "]", &g_cl_procedural_path );
         ap.argString( "plugins", "=string\aOverride the default plugin searchpath(s) [" + g_plugin_path + "]", &g_cl_plugin_path );
-		ap.argInts( "frames", " f1 f2\aSpecify a starting/ending frame to render (inclusive).", &g_cl_frames, ArgParse::SEP_ARGV, 2);
-		ap.argString( "frameslist", "=string\aSpecify a range of frames to render, ',' separated with '-' to indicate ranges.", &g_cl_framesList);
-        ap.argFlag( "nocolor", "\aDisable colored output", &g_cl_no_color );
-        ap.argFlag( "beep", "\aBeep on completion of all ribs", &g_cl_beep );
-        ap.alias( "nocolor", "nc" );
-#ifdef	AQSIS_SYSTEM_POSIX
-        ap.argFlag( "syslog", "\aLog messages to syslog", &g_cl_syslog );
-#endif	// AQSIS_SYSTEM_POSIX
         ap.allowUnrecognizedOptions();
 
         //_crtBreakAlloc = 1305;
