@@ -19,13 +19,17 @@
 
 
 /** \file
-		\brief Simple example display device manager.
+		\brief Compliant display device manager.
 		\author Paul C. Gregory (pgregory@aqsis.com)
 */
 
 
 #include	"aqsis.h"
 
+#ifdef	AQSIS_SYSTEM_WIN32
+#include	"winsock2.h"
+#endif
+
 #include	"sstring.h"
 #include	"ddmanager.h"
 #include	"rifile.h"
@@ -35,17 +39,13 @@
 #include	"ndspy.h"
 #include	"version.h"
 
-#ifdef	AQSIS_SYSTEM_WIN32
-#include	"winsock2.h"
-#endif
-
 START_NAMESPACE( Aqsis )
 
 
 /// Required function that implements Class Factory design pattern for DDManager libraries
 IqDDManager* CreateDisplayDriverManager()
 {
-    return new CqDDManager;
+	return new CqDDManager;
 }
 
 CqString CqDDManager::m_strOpenMethod("DspyImageOpen");
@@ -63,6 +63,8 @@ char* CqDDManager::m_ZName = "z";
 TqInt CqDDManager::AddDisplay( const TqChar* name, const TqChar* type, const TqChar* mode, TqInt modeID, TqInt dataOffset, TqInt dataSize, std::map<std::string, void*> mapOfArguments )
 {
 	SqDisplayRequest req;
+
+	req.m_valid = TqFalse;
 	req.m_name = name;
 	req.m_type = type;
 	req.m_mode = mode;
@@ -75,14 +77,14 @@ TqInt CqDDManager::AddDisplay( const TqChar* name, const TqChar* type, const TqC
 	req.m_QuantizeMinVal = 0.0f;
 	req.m_QuantizeMaxVal = 0.0f;
 	req.m_QuantizeDitherVal = 0.0f;
-	
+
 	// Create the array of UserParameter structures for all the unrecognised extra parameters,
 	// while extracting information for the recognised ones.
 	PrepareCustomParameters(mapOfArguments, req);
 
 	m_displayRequests.push_back(req);
 
-    return ( 0 );
+	return ( 0 );
 }
 
 TqInt CqDDManager::ClearDisplays()
@@ -99,11 +101,11 @@ TqInt CqDDManager::ClearDisplays()
 				free(iup->name);
 				free(iup->value);
 			}
-		}			
+		}
 	}
-	
+
 	m_displayRequests.clear();
-    return ( 0 );
+	return ( 0 );
 }
 
 TqInt CqDDManager::OpenDisplays()
@@ -112,7 +114,7 @@ TqInt CqDDManager::OpenDisplays()
 	std::vector<SqDisplayRequest>::iterator i;
 	for(i = m_displayRequests.begin(); i!=m_displayRequests.end(); i++)
 		LoadDisplayLibrary(*i);
-    return ( 0 );
+	return ( 0 );
 }
 
 TqInt CqDDManager::CloseDisplays()
@@ -121,43 +123,55 @@ TqInt CqDDManager::CloseDisplays()
 	std::vector<SqDisplayRequest>::iterator i;
 	for(i = m_displayRequests.begin(); i!=m_displayRequests.end(); i++)
 		CloseDisplayLibrary(*i);
-    return ( 0 );
+	return ( 0 );
 }
 
 
 
 TqInt CqDDManager::DisplayBucket( IqBucket* pBucket )
 {
-	if( (pBucket->Width() == 0) || (pBucket->Height() == 0) )
+ 	if( (pBucket->Width() == 0) || (pBucket->Height() == 0) )
 		return(0);
 
-    TqInt	xmin = pBucket->XOrigin();
-    TqInt	ymin = pBucket->YOrigin();
-    TqInt	xmaxplus1 = xmin + pBucket->Width();
-    TqInt	ymaxplus1 = ymin + pBucket->Height();
+	TqInt	xmin = pBucket->XOrigin();
+	TqInt	ymin = pBucket->YOrigin();
+	TqInt	xmaxplus1 = xmin + pBucket->Width();
+	TqInt	ymaxplus1 = ymin + pBucket->Height();
 
 	// If completely outside the crop rectangle, don't bother sending.
 	if( xmaxplus1 <= QGetRenderContext()->pImage()->CropWindowXMin() ||
-		ymaxplus1 <= QGetRenderContext()->pImage()->CropWindowYMin() ||
-		xmin > QGetRenderContext()->pImage()->CropWindowXMax() ||
-		ymin > QGetRenderContext()->pImage()->CropWindowYMax() )
+	        ymaxplus1 <= QGetRenderContext()->pImage()->CropWindowYMin() ||
+	        xmin > QGetRenderContext()->pImage()->CropWindowXMax() ||
+	        ymin > QGetRenderContext()->pImage()->CropWindowYMax() )
 		return(0);
 
-    std::vector<SqDisplayRequest>::iterator i;
-    for ( i = m_displayRequests.begin(); i != m_displayRequests.end(); i++ )
-    {
-		// Allocate enough space to put the whole bucket data into.
-		unsigned char* data = reinterpret_cast<unsigned char*>(malloc(i->m_elementSize * pBucket->Width() * pBucket->Height()));
+	std::vector<SqDisplayRequest>::iterator i;
+	for ( i = m_displayRequests.begin(); i != m_displayRequests.end(); i++ )
+	{
+		// If the display is not validated, don't send it data.
+		if( !i->m_valid )
+			continue;
 
-        SqImageSample val( QGetRenderContext()->GetOutputDataTotalSize() );
+		// Allocate enough space to put the whole bucket data into.
+		if (i->m_DataBucket == 0)
+			i->m_DataBucket = reinterpret_cast<unsigned char*>(malloc(i->m_elementSize * pBucket->Width() * pBucket->Height()));
+		if ((i->m_flags.flags & PkDspyFlagsWantsScanLineOrder) && i->m_DataRow == 0)
+		{
+			TqUint width = QGetRenderContext()->pImage()->CropWindowXMax() - QGetRenderContext()->pImage()->CropWindowXMin();
+			TqUint height = pBucket->Height();
+			i->m_DataRow = reinterpret_cast<unsigned char*>(malloc(i->m_elementSize * width * height));
+		}
+
+		SqImageSample val;
 		// Fill in the bucket data for each channel in each element, honoring the requested order and formats.
-		unsigned char* pdata = data;
-        TqInt y;
-        for ( y = ymin; y < ymaxplus1; y++ )
-        {
-            TqInt x;
-            for ( x = xmin; x < xmaxplus1; x++ )
-            {
+		unsigned char* pdata = i->m_DataBucket;
+		TqInt y;
+
+		for ( y = ymin; y < ymaxplus1; y++ )
+		{
+			TqInt x;
+			for ( x = xmin; x < xmaxplus1; x++ )
+			{
 				TqInt index = 0;
 				const TqFloat* pSamples = pBucket->Data( x, y );
 				std::vector<PtDspyDevFormat>::iterator iformat;
@@ -166,163 +180,222 @@ TqInt CqDDManager::DisplayBucket( IqBucket* pBucket )
 					TqFloat value = pSamples[i->m_dataOffsets[index]];
 					// If special quantization instructions have been given for this display, do it now.
 					if( !( i->m_QuantizeZeroVal == 0.0f &&
-						   i->m_QuantizeOneVal  == 0.0f &&
-						   i->m_QuantizeMinVal  == 0.0f &&
-						   i->m_QuantizeMaxVal  == 0.0f ) )
+					        i->m_QuantizeOneVal  == 0.0f &&
+					        i->m_QuantizeMinVal  == 0.0f &&
+					        i->m_QuantizeMaxVal  == 0.0f ) )
 					{
 						value = ROUND(i->m_QuantizeZeroVal + value * (i->m_QuantizeOneVal - i->m_QuantizeZeroVal) + i->m_QuantizeDitherVal );
 						value = CLAMP(value, i->m_QuantizeMinVal, i->m_QuantizeMaxVal) ;
 					}
-					switch(iformat->type)
+					TqInt type = iformat->type & PkDspyMaskType;
+					switch(type)
 					{
-						case PkDspyFloat32:
+							case PkDspyFloat32:
 							reinterpret_cast<float*>(pdata)[0] = value;
 							pdata += sizeof(float);
 							break;
-						case PkDspyUnsigned32:
+							case PkDspyUnsigned32:
 							reinterpret_cast<unsigned long*>(pdata)[0] = static_cast<unsigned long>( value );
 							pdata += sizeof(unsigned long);
 							break;
-						case PkDspySigned32:
+							case PkDspySigned32:
 							reinterpret_cast<long*>(pdata)[0] = static_cast<long>( value );
 							pdata += sizeof(long);
 							break;
-						case PkDspyUnsigned16:
+							case PkDspyUnsigned16:
 							reinterpret_cast<unsigned short*>(pdata)[0] = static_cast<unsigned short>( value );
 							pdata += sizeof(unsigned short);
 							break;
-						case PkDspySigned16:
+							case PkDspySigned16:
 							reinterpret_cast<short*>(pdata)[0] = static_cast<short>( value );
 							pdata += sizeof(short);
 							break;
-						case PkDspyUnsigned8:
+							case PkDspyUnsigned8:
 							reinterpret_cast<unsigned char*>(pdata)[0] = static_cast<unsigned char>( value );
 							pdata += sizeof(unsigned char);
 							break;
-						case PkDspySigned8:
+							case PkDspySigned8:
 							reinterpret_cast<char*>(pdata)[0] = static_cast<char>( value );
 							pdata += sizeof(char);
 							break;
 					}
 					index++;
 				}
-            }
-        }
-		// Now that the bucket data has been constructed, send it to the display.
-		if(i->m_DataMethod )
-		{
-			PtDspyError err = (i->m_DataMethod)(i->m_imageHandle, xmin, xmaxplus1, ymin, ymaxplus1, i->m_elementSize, data);
+			}
 		}
 
-		free(data);
-    }
-    return ( 0 );
+		// Now that the bucket data has been constructed, send it to the display
+		// either lines by lines or bucket by bucket
+		if(i->m_DataMethod )
+		{
+			TqInt elementsize = i->m_elementSize;
+			PtDspyError err;
+
+			if (i->m_flags.flags & PkDspyFlagsWantsScanLineOrder)
+			{
+				TqInt x, y;
+
+				// Accumulate the bucket information to the full row of buckets
+				unsigned char* pdata = i->m_DataBucket;
+				TqUint width = QGetRenderContext()->pImage()->CropWindowXMax() - QGetRenderContext()->pImage()->CropWindowXMin();
+				for (y = ymin; y < ymaxplus1; y++)
+				{
+					for (x = xmin; x < xmaxplus1; x++)
+					{
+						memcpy(&(i->m_DataRow[width * elementsize * (y - ymin) + elementsize * x]), pdata, elementsize);
+						pdata += elementsize;
+					}
+				}
+				pdata = i->m_DataRow;
+
+				// Is it the time sent complete rows
+				if (xmaxplus1 >= width)
+				{
+					// send to the display one line at the time
+					for (y = ymin; y < ymaxplus1; y++)
+					{
+						err = (i->m_DataMethod)(i->m_imageHandle, 0, width, y, y+1, elementsize, pdata);
+						pdata += elementsize * width;
+					}
+				}
+
+			}
+			else
+			{
+				// Send the bucket information as they come in
+				err = (i->m_DataMethod)(i->m_imageHandle, xmin, xmaxplus1, ymin, ymaxplus1, elementsize, i->m_DataBucket);
+
+			}
+
+		}
+
+
+	}
+
+	return ( 0 );
 }
 
 TqBool CqDDManager::fDisplayNeeds( const TqChar* var )
 {
-    static TqUlong rgb = CqString::hash( "rgb" );
-    static TqUlong rgba = CqString::hash( "rgba" );
-    static TqUlong Ci = CqString::hash( "Cs" );
-    static TqUlong Oi = CqString::hash( "Os" );
+	static TqUlong rgb = CqString::hash( "rgb" );
+	static TqUlong rgba = CqString::hash( "rgba" );
+	static TqUlong Ci = CqString::hash( "Ci" );
+	static TqUlong Oi = CqString::hash( "Oi" );
+	static TqUlong Cs = CqString::hash( "Cs" );
+	static TqUlong Os = CqString::hash( "Os" );
+
+	TqUlong htoken = CqString::hash( var );
+
 
-    TqUlong htoken = CqString::hash( var );
 
 	// Scan all registered displays to see if any of them need the variable specified.
 	std::vector<SqDisplayRequest>::iterator i;
 	for(i = m_displayRequests.begin(); i!=m_displayRequests.end(); i++)
-    {
-        TqBool usage = ( ( i->m_modeHash == rgba ) || ( i->m_modeHash == rgb ) );
-        if ( ( htoken == Ci ) && usage )
-            return ( TqTrue );
-        else if ( ( htoken == Oi ) && usage )
-            return ( TqTrue );
-        else if ( ( i->m_modeHash == htoken ) )
-            return ( TqTrue );
-    }
-    return ( TqTrue );
+	{
+		TqBool usage = ( ( i->m_modeHash == rgba ) || ( i->m_modeHash == rgb ) );
+
+		if ( (( htoken == Ci )|| ( htoken == Cs))&& usage )
+			return ( TqTrue );
+		else if ( (( htoken == Oi )|| ( htoken == Os))&& usage )
+			return ( TqTrue );
+		else if ( i->m_modeHash == htoken  )
+			return ( TqTrue );
+	}
+	//printf("fDisplayNeeds(%s) returns false ???\n", var);
+	return ( TqFalse);
 }
 
 TqInt CqDDManager::Uses()
 {
-    TqInt Uses = 0;
+	TqInt Uses = 0;
 	// Scan all registered displays to combine the required variables.
 	std::vector<SqDisplayRequest>::iterator i;
 	for(i = m_displayRequests.begin(); i!=m_displayRequests.end(); i++)
-    {
-        TqInt ivar;
-        for( ivar = 0; ivar < EnvVars_Last; ivar++ )
-        {
-            if( i->m_modeHash == gVariableTokens[ ivar ] )
-                Uses |= 1 << ivar;
-        }
-    }
-    return ( Uses );
+	{
+		TqInt ivar;
+		for( ivar = 0; ivar < EnvVars_Last; ivar++ )
+		{
+			if( i->m_modeHash == gVariableTokens[ ivar ] )
+				Uses |= 1 << ivar;
+		}
+	}
+	return ( Uses );
 }
 
 
 void CqDDManager::LoadDisplayLibrary( SqDisplayRequest& req )
 {
-    if ( !m_fDisplayMapInitialised )
-        InitialiseDisplayNameMap();
+	if ( !m_fDisplayMapInitialised )
+		InitialiseDisplayNameMap();
 
-    // strDriverFileAndArgs: Second part of the ddmsock.ini line (e.g. "mydriver.exe --foo")
-    CqString strDriverFileAndArgs = m_mapDisplayNames[ req.m_type ];
-    // strDriverFile: Only the executable without arguments (e.g. "mydriver.exe")
-    CqString strDriverFile = GetStringField( strDriverFileAndArgs, 0 );
+	// strDriverFileAndArgs: Second part of the ddmsock.ini line (e.g. "mydriver.exe --foo")
+	CqString strDriverFileAndArgs = m_mapDisplayNames[ req.m_type ];
+	// strDriverFile: Only the executable without arguments (e.g. "mydriver.exe")
+	CqString strDriverFile = GetStringField( strDriverFileAndArgs, 0 );
 
 	// Display type not found.
-    if ( strDriverFile.empty() )
-        throw( CqString( "Invalid display type \"" ) + CqString( req.m_type ) + CqString( "\"" ) );
+	if ( strDriverFile.empty() )
+		throw( CqString( "Invalid display type \"" ) + CqString( req.m_type ) + CqString( "\"" ) );
 
-    // Try to open the file to see if it's really there
-    CqRiFile fileDriver( strDriverFile.c_str(), "display" );
+	// Try to open the file to see if it's really there
+	CqRiFile fileDriver( strDriverFile.c_str(), "display" );
 
-    if ( !fileDriver.IsValid() )
-        throw( CqString( "Error loading display driver [ " ) + strDriverFile + CqString( " ]" ) );
+	if ( !fileDriver.IsValid() )
+		throw( CqString( "Error loading display driver [ " ) + strDriverFile + CqString( " ]" ) );
 
-    CqString strDriverPathAndFile = fileDriver.strRealName();
+	CqString strDriverPathAndFile = fileDriver.strRealName();
 
 	// Load the dynamic obejct and locate the relevant symbols.
-    req.m_DriverHandle = m_DspyPlugin.SimpleDLOpen( &strDriverPathAndFile );
-    if( req.m_DriverHandle != NULL )
-    {
-        req.m_OpenMethod = (DspyImageOpenMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strOpenMethod );
-       if (!req.m_OpenMethod) {
-          m_strOpenMethod = "_" + m_strOpenMethod;
-          req.m_OpenMethod = (DspyImageOpenMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strOpenMethod );
-       }
-       req.m_QueryMethod = (DspyImageQueryMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strQueryMethod );
-       if (!req.m_QueryMethod) {
-          m_strQueryMethod = "_" + m_strQueryMethod;
-        req.m_QueryMethod = (DspyImageQueryMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strQueryMethod );
-       }
-       req.m_DataMethod = (DspyImageDataMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDataMethod );
-       if (!req.m_DataMethod) {
-          m_strDataMethod = "_" + m_strDataMethod;
-        req.m_DataMethod = (DspyImageDataMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDataMethod );
-       }
-       req.m_CloseMethod = (DspyImageCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strCloseMethod );
-       if (!req.m_OpenMethod) {
-          m_strCloseMethod = "_" + m_strCloseMethod;
-        req.m_CloseMethod = (DspyImageCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strCloseMethod );
-       }
-       req.m_DelayCloseMethod = (DspyImageDelayCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDelayCloseMethod );
-       if (!req.m_DelayCloseMethod) {
-          m_strDelayCloseMethod = "_" + m_strDelayCloseMethod;
-        req.m_DelayCloseMethod = (DspyImageDelayCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDelayCloseMethod );
-	}
+	req.m_DriverHandle = m_DspyPlugin.SimpleDLOpen( &strDriverPathAndFile );
+	if( req.m_DriverHandle != NULL )
+	{
+		req.m_OpenMethod = (DspyImageOpenMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strOpenMethod );
+		if (!req.m_OpenMethod)
+		{
+			m_strOpenMethod = "_" + m_strOpenMethod;
+			req.m_OpenMethod = (DspyImageOpenMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strOpenMethod );
+		}
+		req.m_QueryMethod = (DspyImageQueryMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strQueryMethod );
+		if (!req.m_QueryMethod)
+		{
+			m_strQueryMethod = "_" + m_strQueryMethod;
+			req.m_QueryMethod = (DspyImageQueryMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strQueryMethod );
+		}
+		req.m_DataMethod = (DspyImageDataMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDataMethod );
+		if (!req.m_DataMethod)
+		{
+			m_strDataMethod = "_" + m_strDataMethod;
+			req.m_DataMethod = (DspyImageDataMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDataMethod );
+		}
+		req.m_CloseMethod = (DspyImageCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strCloseMethod );
+		if (!req.m_OpenMethod)
+		{
+			m_strCloseMethod = "_" + m_strCloseMethod;
+			req.m_CloseMethod = (DspyImageCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strCloseMethod );
+		}
+		req.m_DelayCloseMethod = (DspyImageDelayCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDelayCloseMethod );
+		if (!req.m_DelayCloseMethod)
+		{
+			m_strDelayCloseMethod = "_" + m_strDelayCloseMethod;
+			req.m_DelayCloseMethod = (DspyImageDelayCloseMethod)m_DspyPlugin.SimpleDLSym( req.m_DriverHandle, &m_strDelayCloseMethod );
+		}
 	}
 
-    if( NULL != req.m_OpenMethod )
-    {
+	// Nullified the data part
+	req.m_DataRow = 0;
+	req.m_DataBucket = 0;
+
+	if( NULL != req.m_OpenMethod )
+	{
 		TqFloat colorQuantOne = 255.0f;
 		TqFloat depthQuantOne = 0.0f;
 		// Get some key information about the render to be used when initialising the display.
-	    const TqFloat* pQuant = QGetRenderContext() ->optCurrent().GetFloatOption( "Quantize", "Color" );
-		if( pQuant )	colorQuantOne = pQuant[0];
-	    pQuant = QGetRenderContext() ->optCurrent().GetFloatOption( "Quantize", "Depth" );
-		if( pQuant )	depthQuantOne = pQuant[0];
+		const TqFloat* pQuant = QGetRenderContext() ->optCurrent().GetFloatOption( "Quantize", "Color" );
+		if( pQuant )
+			colorQuantOne = pQuant[0];
+		pQuant = QGetRenderContext() ->optCurrent().GetFloatOption( "Quantize", "Depth" );
+		if( pQuant )
+			depthQuantOne = pQuant[0];
 
 		// Prepare the information and call the DspyImageOpen function in the display device.
 		if(req.m_modeID & ( ModeRGB | ModeA | ModeZ) )
@@ -334,7 +407,8 @@ void CqDDManager::LoadDisplayLibrary( SqDisplayRequest& req )
 				fmt.type = PkDspyUnsigned16;
 			else if( colorQuantOne == 4294967295u )
 				fmt.type = PkDspyUnsigned32;
-			else fmt.type = PkDspyFloat32;
+			else
+				fmt.type = PkDspyFloat32;
 			if(req.m_modeID & ModeA)
 			{
 				fmt.name = m_AlphaName;
@@ -362,20 +436,16 @@ void CqDDManager::LoadDisplayLibrary( SqDisplayRequest& req )
 			// Determine the type of the AOV data being displayed.
 			TqInt type;
 			type = QGetRenderContext()->OutputDataType(req.m_mode.c_str());
-			// Generate a suitable name for the channels.
-			// We use <variable name>.<channel indec>.<component name>
-			// where <component name> is r,g,b for color, x,y,z for point, otherwise nothing.
-			std::string baseName = req.m_mode;
 			std::string componentNames = "";
 			switch(type)
 			{
-				case type_point:
-				case type_normal:
-				case type_vector:
-				case type_hpoint:
-					componentNames = "xyz";
+					case type_point:
+					case type_normal:
+					case type_vector:
+					case type_hpoint:
+					componentNames = "XYZ";
 					break;
-				case type_color:
+					case type_color:
 					componentNames = "rgb";
 					break;
 			}
@@ -384,26 +454,49 @@ void CqDDManager::LoadDisplayLibrary( SqDisplayRequest& req )
 			TqInt i;
 			for( i = 0; i < req.m_AOVSize; i++ )
 			{
-				std::string name(baseName);
-				name.append(".");
-				name.append(ToString(i));
 				if(componentNames.size()>i)
 				{
-					name.append(".");
-					name.append(componentNames.substr(i, 1));
-				}
-				req.m_AOVnames.push_back(name);
-				fmt.name = const_cast<char*>(req.m_AOVnames.back().c_str());
-				fmt.type = PkDspyFloat32;
 
+					if (componentNames.substr(i, 1) == "r")
+						fmt.name = m_RedName;
+					if (componentNames.substr(i, 1) == "g")
+						fmt.name = m_GreenName;
+					if (componentNames.substr(i, 1) == "b")
+						fmt.name = m_BlueName;
+					if (componentNames.substr(i, 1) == "a")
+						fmt.name = m_AlphaName;
+					if (componentNames.substr(i, 1) == "z")
+						fmt.name = m_ZName;
+					if (componentNames.substr(i, 1) == "X")
+						fmt.name = m_RedName;
+					if (componentNames.substr(i, 1) == "Y")
+						fmt.name = m_GreenName;
+					if (componentNames.substr(i, 1) == "Z")
+						fmt.name = m_BlueName;
+
+				}
+				else
+				{
+					// by default we will stored into red channel eg. "s" will be saved into 'r' channel
+					fmt.name = m_RedName;
+				}
+				if( colorQuantOne == 255 )
+					fmt.type = PkDspyUnsigned8;
+				else if( colorQuantOne == 65535 )
+					fmt.type = PkDspyUnsigned16;
+				else if( colorQuantOne == 4294967295u )
+					fmt.type = PkDspyUnsigned32;
+				else
+					fmt.type = PkDspyFloat32;
+				req.m_AOVnames.push_back(fmt.name);
 				req.m_formats.push_back(fmt);
-			}		
+			}
 		}
 
 		// If we got here, we are dealing with a valid display device, so now is the time
 		// to fill in the system parameters.
 		PrepareSystemParameters(req);
-        
+
 		// Call the DspyImageOpen method on the display to initialise things.
 		TqInt xres = QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "Resolution" ) [ 0 ];
 		TqInt yres = QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "Resolution" ) [ 1 ];
@@ -411,14 +504,46 @@ void CqDDManager::LoadDisplayLibrary( SqDisplayRequest& req )
 		TqInt xmax = static_cast<TqInt>( CLAMP( CEIL( xres * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 1 ] ), 0, xres ) );
 		TqInt ymin = static_cast<TqInt>( CLAMP( CEIL( yres * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 2 ] ), 0, yres ) );
 		TqInt ymax = static_cast<TqInt>( CLAMP( CEIL( yres * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 3 ] ), 0, yres ) );
-		PtDspyError err = (*req.m_OpenMethod)(&req.m_imageHandle, 
-											  req.m_type.c_str(), req.m_name.c_str(), 
-											  xmax-xmin, 
-											  ymax-ymin, 
-											  req.m_customParams.size(), 
-											  &req.m_customParams[0], 
-											  req.m_formats.size(), &req.m_formats[0], 
-											  &req.m_flags);
+		PtDspyError err = (*req.m_OpenMethod)(&req.m_imageHandle,
+		                                      req.m_type.c_str(), req.m_name.c_str(),
+		                                      xmax-xmin,
+		                                      ymax-ymin,
+		                                      req.m_customParams.size(),
+		                                      &req.m_customParams[0],
+		                                      req.m_formats.size(), &req.m_formats[0],
+		                                      &req.m_flags);
+
+		// Check for an error
+		if( err != PkDspyErrorNone )
+		{
+			// The display did not successfully open, so clean up after it and leave the 
+			// request as invalid.
+			std::cerr << error << "Cannot open display \"" << req.m_name << "\" : ";
+			switch(err)
+			{
+				case PkDspyErrorNoMemory:
+					std::cerr << "Out of memory" << std::endl;
+					break;
+				case PkDspyErrorUnsupported:
+					std::cerr << "Unsupported" << std::endl;
+					break;
+				case PkDspyErrorBadParams:
+					std::cerr << "Bad params" << std::endl;
+					break;
+				case PkDspyErrorNoResource:
+					std::cerr << "No resource" << std::endl;
+					break;
+				case PkDspyErrorUndefined:			
+				default:
+					std::cerr << "Undefined" << std::endl;
+					break;
+			}
+
+			CloseDisplayLibrary(req);
+			return;			
+		}
+		else
+			req.m_valid = TqTrue;
 
 		// Now scan the returned format list to make sure that we pass the data in the order the display wants it.
 		std::vector<PtDspyDevFormat>::iterator i;
@@ -442,9 +567,10 @@ void CqDDManager::LoadDisplayLibrary( SqDisplayRequest& req )
 				// Scan through the generated names to find the ones specified, and use the index
 				// of the found name as an offset into the data from the dataOffset passed in originally.
 				TqInt iname;
+
 				for(iname = 0; iname < req.m_AOVnames.size(); iname++)
 				{
-					if(i->name == req.m_AOVnames[iname].c_str())
+					if(i->name == req.m_AOVnames[iname])
 					{
 						req.m_dataOffsets.push_back(req.m_AOVOffset + iname );
 						break;
@@ -464,34 +590,38 @@ void CqDDManager::LoadDisplayLibrary( SqDisplayRequest& req )
 		std::vector<PtDspyDevFormat>::iterator iformat;
 		for(iformat = req.m_formats.begin(); iformat != req.m_formats.end(); iformat++)
 		{
-			switch(iformat->type)
+			TqInt type = iformat->type & PkDspyMaskType;
+			switch( type )
 			{
-				case PkDspyFloat32:
+					case PkDspyFloat32:
 					req.m_elementSize+=sizeof(float);
 					break;
-				case PkDspyUnsigned32:
-				case PkDspySigned32:
+					case PkDspyUnsigned32:
+					case PkDspySigned32:
 					req.m_elementSize+=sizeof(long);
 					break;
-				case PkDspyUnsigned16:
-				case PkDspySigned16:
+					case PkDspyUnsigned16:
+					case PkDspySigned16:
 					req.m_elementSize+=sizeof(short);
 					break;
-				case PkDspyUnsigned8:
-				case PkDspySigned8:
+					case PkDspyUnsigned8:
+					case PkDspySigned8:
 					req.m_elementSize+=sizeof(char);
 					break;
 			}
 		}
 
+		//std::cerr << warning << "Elementsize will be " << req.m_elementSize << std::endl;
 		if( NULL != req.m_QueryMethod )
 		{
 			PtDspySizeInfo size;
 			err = (*req.m_QueryMethod)(req.m_imageHandle, PkSizeQuery, sizeof(size), &size);
 			PtDspyOverwriteInfo owinfo;
+			owinfo.interactive = 0;
+			owinfo.overwrite = 1;
 			err = (*req.m_QueryMethod)(req.m_imageHandle, PkOverwriteQuery, sizeof(owinfo), &owinfo);
 		}
-    }
+	}
 }
 
 void CqDDManager::CloseDisplayLibrary( SqDisplayRequest& req )
@@ -503,8 +633,28 @@ void CqDDManager::CloseDisplayLibrary( SqDisplayRequest& req )
 	else if( NULL != req.m_CloseMethod )
 		(*req.m_CloseMethod)(req.m_imageHandle);
 
-	/// \note We don't close the driver shared libraries here because doing so caused 
-	/// some problems with Win2K and FLTK. It seems that detatching from the drive DLL 
+	if (req.m_DataBucket != 0)
+	{
+		free(req.m_DataBucket);
+		req.m_DataBucket = 0;
+	}
+	if (req.m_DataRow != 0)
+	{
+		free(req.m_DataRow);
+		req.m_DataRow = 0;
+	}
+
+	// Empty out the display request data
+	req.m_CloseMethod = NULL;
+	req.m_DataMethod = NULL;
+	req.m_DelayCloseMethod = NULL;
+	req.m_DriverHandle = 0;
+	req.m_imageHandle = 0;
+	req.m_OpenMethod = NULL;
+	req.m_QueryMethod = NULL;
+
+	/// \note We don't close the driver shared libraries here because doing so caused
+	/// some problems with Win2K and FLTK. It seems that detaching from the drive DLL
 	/// causes some important data to be altered and when a new window is opened it crashes.
 	/// The cleanup of the drivers is left to when the CqDDManager instance closes down, and the
 	/// CqSimplePlugin class gets destroyed, which will be at the end of the render, which is fine.
@@ -512,74 +662,74 @@ void CqDDManager::CloseDisplayLibrary( SqDisplayRequest& req )
 
 void CqDDManager::InitialiseDisplayNameMap()
 {
-/*
-    // Read in the configuration file.
-    // Find the config file in the same place as the display drivers.
-#ifdef AQSIS_SYSTEM_POSIX
-    CqString ddmsock_path( "" );
-    char* env = NULL;
-
-    env = getenv( "AQSIS_CONFIG_PATH" );
-    if ( env == NULL )
-    {
-#ifndef __apple__
-        ddmsock_path = CONFIG_PATH;
-#else
-        ddmsock_path = ".";
-#endif
-
-    }
-    else
-    {
-        ddmsock_path = env;
-    }
-
-    ddmsock_path.append( "/" );
-
-    ddmsock_path.append( "displays.ini" );
-
-    CqString strConfigFile = ddmsock_path;
-#else
-    CqString strConfigFile = "displays.ini";
-#endif // AQSIS_SYSTEM_POSIX 
-*/
+	/*
+	    // Read in the configuration file.
+	    // Find the config file in the same place as the display drivers.
+	#ifdef AQSIS_SYSTEM_POSIX
+	    CqString ddmsock_path( "" );
+	    char* env = NULL;
+	 
+	    env = getenv( "AQSIS_CONFIG_PATH" );
+	    if ( env == NULL )
+	    {
+	#ifndef __apple__
+	        ddmsock_path = CONFIG_PATH;
+	#else
+	        ddmsock_path = ".";
+	#endif
+	 
+	    }
+	    else
+	    {
+	        ddmsock_path = env;
+	    }
+	 
+	    ddmsock_path.append( "/" );
+	 
+	    ddmsock_path.append( "displays.ini" );
+	 
+	    CqString strConfigFile = ddmsock_path;
+	#else
+	    CqString strConfigFile = "displays.ini";
+	#endif // AQSIS_SYSTEM_POSIX 
+	*/
 
 
-    CqString strConfigFile("displays.ini");
-    const CqString* displays = QGetRenderContext()->optCurrent().GetStringOption( "searchpath", "displays" );
-    if( displays )
-                strConfigFile = displays[ 0 ] + "/" + strConfigFile;
+	CqString strConfigFile("displays.ini");
+	const CqString* displays = QGetRenderContext()->optCurrent().GetStringOption( "searchpath", "displays" );
+	if( displays )
+		strConfigFile = displays[ 0 ] + "/" + strConfigFile;
 
-    std::cerr << info << "Loading display configuration from file \"" << strConfigFile << "\"" << std::endl;
+	std::cerr << info << "Loading display configuration from file \"" << strConfigFile << "\"" << std::endl;
 
-    CqRiFile fileINI( strConfigFile.c_str(), "display" );
-    if ( fileINI.IsValid() )
-    {
-        // On each line, read the first string, then the second and store them in the map.
-        std::string strLine;
-        std::istream& strmINI = static_cast<std::istream&>( fileINI );
+	CqRiFile fileINI( strConfigFile.c_str(), "display" );
+	if ( fileINI.IsValid() )
+	{
+		// On each line, read the first string, then the second and store them in the map.
+		std::string strLine;
+		std::istream& strmINI = static_cast<std::istream&>( fileINI );
 
-        while ( std::getline( strmINI, strLine ) )
-        {
-            std::string strName, strDriverName;
-            std::string::size_type iStartN = strLine.find_first_not_of( "\t " );
-            std::string::size_type iEndN = strLine.find_first_of( "\t ", iStartN );
-            std::string::size_type iStartD = strLine.find_first_not_of( "\t ", iEndN );
-            std::string::size_type iEndD = strLine.find_first_of( "\t ", iStartD );
-            if ( iStartN != std::string::npos && iEndN != std::string::npos &&
-                    iStartD != std::string::npos )
-            {
-                strName = strLine.substr( iStartN, iEndN );
-                strDriverName = strLine.substr( iStartD, iEndD );
-                m_mapDisplayNames[ strName ] = strDriverName;
-            }
-        }
-        m_fDisplayMapInitialised = true;
-    }
-    else
-    {
-        std::cerr << error << "Could not find " << strConfigFile << " configuration file." << std::endl;
-    }
+		while ( std::getline( strmINI, strLine ) )
+		{
+			std::string strName, strDriverName;
+			std::string::size_type iStartN = strLine.find_first_not_of( "\t " );
+			std::string::size_type iEndN = strLine.find_first_of( "\t ", iStartN );
+			std::string::size_type iStartD = strLine.find_first_not_of( "\t ", iEndN );
+			std::string::size_type iEndD = strLine.find_first_of( "\t ", iStartD );
+			if ( iStartN != std::string::npos && iEndN != std::string::npos &&
+			        iStartD != std::string::npos )
+			{
+				strName = strLine.substr( iStartN, iEndN );
+				strDriverName = strLine.substr( iStartD, iEndD );
+				m_mapDisplayNames[ strName ] = strDriverName;
+			}
+		}
+		m_fDisplayMapInitialised = true;
+	}
+	else
+	{
+		std::cerr << error << "Could not find " << strConfigFile << " configuration file." << std::endl;
+	}
 }
 
 /**
@@ -596,52 +746,56 @@ void CqDDManager::InitialiseDisplayNameMap()
 */
 std::string CqDDManager::GetStringField( const std::string& s, int idx )
 {
-    int z = 1;   /* state variable  0=skip whitespace  1=skip chars  2=search end  3=end */
-    std::string::const_iterator it;
-    std::string::size_type start = 0;
-    std::string::size_type end = 0;
+	int z = 1;   /* state variable  0=skip whitespace  1=skip chars  2=search end  3=end */
+	std::string::const_iterator it;
+	std::string::size_type start = 0;
+	std::string::size_type end = 0;
 
-    for ( it = s.begin(); it != s.end(); it++ )
-    {
-        char c = *it;
+	for ( it = s.begin(); it != s.end(); it++ )
+	{
+		char c = *it;
 
-        if ( idx == 0 && z < 2 )
-        {
-            z = 2;
-        }
+		if ( idx == 0 && z < 2 )
+		{
+			z = 2;
+		}
 
-        switch ( z )
-        {
-        case 0: if ( c != ' ' && c != '\t' )
-            {
-                idx--;
-                end = start + 1;
-                z = 1;
-            }
-            if ( idx > 0 ) start++;
-            break;
-        case 1: if ( c == ' ' || c == '\t' )
-            {
-                z = 0;
-            }
-            start++;
-            break;
-        case 2: if ( c == ' ' || c == '\t' )
-            {
-                z = 3;
-            }
-            else
-            {
-                end++;
-            }
-            break;
-        }
-    }
+		switch ( z )
+		{
+				case 0:
+				if ( c != ' ' && c != '\t' )
+				{
+					idx--;
+					end = start + 1;
+					z = 1;
+				}
+				if ( idx > 0 )
+					start++;
+				break;
+				case 1:
+				if ( c == ' ' || c == '\t' )
+				{
+					z = 0;
+				}
+				start++;
+				break;
+				case 2:
+				if ( c == ' ' || c == '\t' )
+				{
+					z = 3;
+				}
+				else
+				{
+					end++;
+				}
+				break;
+		}
+	}
 
-    if ( idx == 0 )
-        return s.substr( start, end - start );
-    else
-        return std::string( "" );
+	if ( idx == 0 )
+		return s.substr( start, end - start );
+	else
+		return std::string( "" );
 
 }
 
@@ -738,8 +892,8 @@ void CqDDManager::PrepareCustomParameters( std::map<std::string, void*>& mapPara
 {
 	// Scan the map of extra parameters
 	std::map<std::string, void*>::iterator param;
-    for ( param = mapParams.begin(); param != mapParams.end(); param++ )
-    {
+	for ( param = mapParams.begin(); param != mapParams.end(); param++ )
+	{
 		// First check if it is one of the recognised parameters that the renderer should handle.
 		if(param->first.compare("quantize")==0)
 		{
@@ -791,30 +945,30 @@ void CqDDManager::PrepareCustomParameters( std::map<std::string, void*>& mapPara
 
 			switch ( Decl.m_Type )
 			{
-				case type_string:
-				{
-					const char** strings = static_cast<const char**>( param->second );
-					ConstructStringsParameter(Decl.m_strName.c_str(), strings, Decl.m_Count, parameter);
-				}
-				break;
+					case type_string:
+					{
+						const char** strings = static_cast<const char**>( param->second );
+						ConstructStringsParameter(Decl.m_strName.c_str(), strings, Decl.m_Count, parameter);
+					}
+					break;
 
-				case type_float:
-				{
-					const RtFloat* floats = static_cast<RtFloat*>( param->second );
-					ConstructFloatsParameter(Decl.m_strName.c_str(), floats, Decl.m_Count, parameter);
-				}
-				break;
+					case type_float:
+					{
+						const RtFloat* floats = static_cast<RtFloat*>( param->second );
+						ConstructFloatsParameter(Decl.m_strName.c_str(), floats, Decl.m_Count, parameter);
+					}
+					break;
 
-				case type_integer:
-				{
-					const RtInt* ints = static_cast<RtInt*>( param->second );
-					ConstructIntsParameter(Decl.m_strName.c_str(), ints, Decl.m_Count, parameter);
-				}
-				break;
+					case type_integer:
+					{
+						const RtInt* ints = static_cast<RtInt*>( param->second );
+						ConstructIntsParameter(Decl.m_strName.c_str(), ints, Decl.m_Count, parameter);
+					}
+					break;
 			}
 			req.m_customParams.push_back(parameter);
 		}
-    }
+	}
 }
 
 
@@ -825,36 +979,36 @@ void CqDDManager::PrepareSystemParameters( SqDisplayRequest& req )
 	UserParameter parameter;
 
 	// "NP"
-    CqMatrix matWorldToScreen = QGetRenderContext() ->matSpaceToSpace( "world", "screen", CqMatrix(), CqMatrix(), QGetRenderContextI()->Time() );
+	CqMatrix matWorldToScreen = QGetRenderContext() ->matSpaceToSpace( "world", "screen", CqMatrix(), CqMatrix(), QGetRenderContextI()->Time() );
 	ConstructMatrixParameter("NP", &matWorldToScreen, 1, parameter);
 	req.m_customParams.push_back(parameter);
 
 	// "Nl"
-    CqMatrix matWorldToCamera = QGetRenderContext() ->matSpaceToSpace( "world", "camera", CqMatrix(), CqMatrix(), QGetRenderContextI()->Time() );
+	CqMatrix matWorldToCamera = QGetRenderContext() ->matSpaceToSpace( "world", "camera", CqMatrix(), CqMatrix(), QGetRenderContextI()->Time() );
 	ConstructMatrixParameter("Nl", &matWorldToCamera, 1, parameter);
 	req.m_customParams.push_back(parameter);
 
 	// "near"
-    TqFloat nearval = static_cast<TqFloat>( QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 0 ] );
+	TqFloat nearval = static_cast<TqFloat>( QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 0 ] );
 	ConstructFloatsParameter("near", &nearval, 1, parameter);
 	req.m_customParams.push_back(parameter);
 
 	// "far"
-    TqFloat farval = static_cast<TqFloat>( QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 1 ] );
+	TqFloat farval = static_cast<TqFloat>( QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 1 ] );
 	ConstructFloatsParameter("far", &farval, 1, parameter);
 	req.m_customParams.push_back(parameter);
 
 	// "OriginalSize"
-    TqInt OriginalSize[2];
+	TqInt OriginalSize[2];
 	OriginalSize[0] = QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "Resolution" ) [ 0 ];
 	OriginalSize[1] = QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "Resolution" ) [ 1 ];
 	ConstructIntsParameter("OriginalSize", OriginalSize, 2, parameter);
 	req.m_customParams.push_back(parameter);
 
 	// "origin"
-    TqInt origin[2];
-    origin[0] = static_cast<TqInt>( CLAMP( CEIL( OriginalSize[0] * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 0 ] ), 0, OriginalSize[0] ) );
-    origin[1] = static_cast<TqInt>( CLAMP( CEIL( OriginalSize[1] * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 2 ] ), 0, OriginalSize[1] ) );
+	TqInt origin[2];
+	origin[0] = static_cast<TqInt>( CLAMP( CEIL( OriginalSize[0] * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 0 ] ), 0, OriginalSize[0] ) );
+	origin[1] = static_cast<TqInt>( CLAMP( CEIL( OriginalSize[1] * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 2 ] ), 0, OriginalSize[1] ) );
 	ConstructIntsParameter("origin", origin, 2, parameter);
 	req.m_customParams.push_back(parameter);
 
@@ -864,9 +1018,9 @@ void CqDDManager::PrepareSystemParameters( SqDisplayRequest& req )
 	req.m_customParams.push_back(parameter);
 
 	// "Software"
-    char SoftwareName[ 80 ];
+	char SoftwareName[ 80 ];
 	const char* Software = SoftwareName;
-    sprintf( SoftwareName, "%s %s", STRNAME, VERSION_STR );
+	sprintf( SoftwareName, "%s %s (%s %s)", STRNAME, VERSION_STR, __DATE__, __TIME__ );
 	ConstructStringsParameter("Software", &Software, 1, parameter);
 	req.m_customParams.push_back(parameter);
 
@@ -874,13 +1028,17 @@ void CqDDManager::PrepareSystemParameters( SqDisplayRequest& req )
 	char HostComputerName[255];
 	const char* HostComputer = HostComputerName;
 #ifdef AQSIS_SYSTEM_WIN32
+
 	WSADATA wsaData;
 	WSAStartup( MAKEWORD( 2, 0 ), &wsaData );
 #endif // AQSIS_SYSTEM_WIN32
+
 	gethostname( HostComputerName, 255 );
 #ifdef	AQSIS_SYSTEM_WIN32
+
 	WSACleanup();
 #endif
+
 	ConstructStringsParameter("HostComputer", &HostComputer, 1, parameter);
 	req.m_customParams.push_back(parameter);
 }
