@@ -67,7 +67,7 @@ protected:
                       Torus, Blobby, Points, Curves,
                       SubdivisionMesh, Procedural, Geometry, MakeTexture,
                       MakeBump, MakeLatLongEnvironment, MakeCubeFaceEnvironment, MakeShadow,
-                      ArchiveRecord, ReadArchive, ErrorHandler, LAST_Function };
+                      ArchiveRecord, ReadArchive, ErrorHandler, IfBegin, IfEnd, Else, ElseIf, LAST_Function };
     // block types (used to check context nesting)
     enum EqBlocks{
         B_Ri, B_Frame, B_World, B_Attribute, B_Transform, B_Solid, B_Object, 
@@ -331,6 +331,12 @@ public:
 
     /* ERROR HANDLER */
     RtVoid RiErrorHandler( RtErrorFunc handler );
+
+    /* 3.04 New Condition statements */
+    RtVoid RiIfBegin( RtString condition );
+    RtVoid RiIfEnd( );
+    RtVoid RiElse();
+    RtVoid RiElseIf( RtString condition);
 };
 
 
