@@ -25,6 +25,8 @@
 
 #include	"aqsis.h"
 
+#include	"MultiTimer.h"
+
 #include	<iostream>
 
 #include	<sstream>
@@ -403,6 +405,8 @@ SqOpCodeTrans CqShaderVM::m_TransTable[] =
         {"filterstep2", 0, &CqShaderVM::SO_filterstep2, 0, {0}},
         {"specularbrdf", 0, &CqShaderVM::SO_specularbrdf, 0, {0}},
 
+        {"mtransform2", 0, &CqShaderVM::SO_mtransform2, 0, {0}},
+        {"mtransform", 0, &CqShaderVM::SO_mtransform, 0, {0}},
         {"mcomp", 0, &CqShaderVM::SO_mcomp, 0, {0}},
         {"setmcomp", 0, &CqShaderVM::SO_setmcomp, 0, {0}},
         {"determinant", 0, &CqShaderVM::SO_determinant, 0, {0}},
@@ -1248,13 +1252,10 @@ CqShaderVM&	CqShaderVM::operator=( const CqShaderVM& From )
         m_LocalVars.push_back( ( *i ) ->Clone() );
 
     // Copy the intialisation program.
-    std::vector<UsProgramElement>::const_iterator p;
-    for ( p = From.m_ProgramInit.begin(); p != From.m_ProgramInit.end(); p++ )
-        m_ProgramInit.push_back( *p );
+	m_ProgramInit.assign(From.m_ProgramInit.begin(), From.m_ProgramInit.end());
 
     // Copy the main program.
-    for ( p = From.m_Program.begin(); p != From.m_Program.end(); p++ )
-        m_Program.push_back( *p );
+	m_Program.assign(From.m_Program.begin(), From.m_Program.end());
 
     return ( *this );
 }
@@ -1487,7 +1488,8 @@ void CqShaderVM::SetArgument( CqParameter* pParam, IqSurface* pSurface )
     {
         /// \todo: Find out how to handle arrays.
         IqShaderData* pVar = m_LocalVars[ i ];
-        pParam->Dice(m_uGridRes,m_vGridRes,pVar,pSurface);
+		if(pVar->Type() == pParam->Type())
+			pParam->Dice(m_uGridRes,m_vGridRes,pVar,pSurface);
     }
 }
 
@@ -1511,7 +1513,7 @@ IqShaderData* CqShaderVM::FindArgument( const CqString& name )
 /** Get a value from an instance variable on this shader, and fill in the passed variable reference.
 */
 
-TqBool CqShaderVM::GetValue( const char* name, IqShaderData* res )
+TqBool CqShaderVM::GetVariableValue( const char* name, IqShaderData* res )
 {
     // Find the relevant variable.
     TqInt i = FindLocalVarIndex( name );
