@@ -543,7 +543,7 @@ public:
     virtual	void	SetArgument( const CqString& name, EqVariableType type, const CqString& space, void* val );
     virtual	void	SetArgument( CqParameter* pParam, IqSurface* pSurface );
     virtual	IqShaderData*	FindArgument( const CqString& name );
-    virtual	TqBool	GetValue( const char* name, IqShaderData* res );
+    virtual	TqBool	GetVariableValue( const char* name, IqShaderData* res );
     virtual	void	Evaluate( const boost::shared_ptr<IqShaderExecEnv>& pEnv )
     {
         Execute( pEnv );
@@ -560,7 +560,6 @@ public:
     virtual	IqShader*	Clone() const
     {
         CqShaderVM * pShader = new CqShaderVM( *this );
-        pShader->ExecuteInit();
         return ( pShader );
     }
     virtual TqBool	Uses( TqInt Var ) const
@@ -577,6 +576,14 @@ public:
     virtual	IqShaderData* CreateTemporaryStorage( EqVariableType type, EqVariableClass _class );
     virtual void DeleteTemporaryStorage( IqShaderData* pData );
     virtual void DefaultSurface();
+	virtual	TqBool IsLayered()
+	{
+		return(TqFalse);
+	}
+	void AddLayer(const CqString& layername, const boost::shared_ptr<IqShader>& layer)
+	{}
+	virtual void AddConnection(const CqString& layer1, const CqString& variable1, const CqString& layer2, const CqString& variable2)
+	{}
 
     /// \todo: These should be exposed by the IqShader Interface somehow.
     static	void ShutdownShaderEngine();
@@ -897,6 +904,8 @@ private:
     void	SO_ntransform2();
     void	SO_ntransform();
     void	SO_ntransformm();
+    void	SO_mtransform2();
+    void	SO_mtransform();
     void	SO_depth();
     void	SO_calculatenormal();
     void	SO_comp();
