@@ -246,10 +246,6 @@ CqFuncDef	gStandardFuncs[] = {
                                  CqFuncDef( Type_Point, "cellnoise", "pcellnoise2", "ff" ),
                                  CqFuncDef( Type_Point, "cellnoise", "pcellnoise3", "p" ),
                                  CqFuncDef( Type_Point, "cellnoise", "pcellnoise4", "pf" ),
-				 CqFuncDef( Type_Vector, "cellnoise", "pcellnoise1", "f" ),
-				 CqFuncDef( Type_Vector, "cellnoise", "pcellnoise2", "ff" ),
-				 CqFuncDef( Type_Vector, "cellnoise", "pcellnoise3", "p" ),
-				 CqFuncDef( Type_Vector, "cellnoise", "pcellnoise4", "pf" ),
                                  CqFuncDef( Type_Float, "mix", "fmix", "fff" ),
                                  CqFuncDef( Type_Point, "mix", "pmix", "ppf" ),
                                  CqFuncDef( Type_Vector, "mix", "vmix", "vvf" ),
