@@ -178,7 +178,7 @@ typedef union YYSTYPE {
 	EqCommType		m_CommType;
 } YYSTYPE;
 /* Line 1240 of yacc.c.  */
-#line 181 "y.tab.h"
+#line 181 "parser.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
