@@ -9,6 +9,8 @@
 #include	"aqsis.h"
 #include	"parsenode.h"
 
+#include	<queue>
+
 START_NAMESPACE( Aqsis )
 
 ///---------------------------------------------------------------------
@@ -16,13 +18,60 @@ START_NAMESPACE( Aqsis )
 /// Do a type check based on suitable types requested, and add a cast if required.
 /// Uses all possible functions listed to try to fund a suitable candidate.
 
-TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+class CqFunctionSignature
+{
+	public:
+		CqFunctionSignature(SqFuncRef ref, CqFuncDef* func, TqInt weight, TqBool mustCast) : m_ref(ref), m_func(func), m_weight(weight), m_mustCast(mustCast)
+		{}
+		~CqFunctionSignature() {}
+
+		const SqFuncRef& getRef() const
+		{
+			return(m_ref);
+		}
+
+		SqFuncRef& getRef()
+		{
+			return(m_ref);
+		}
+
+		CqFuncDef* getFunc() const
+		{
+			return(m_func);
+		}
+
+		TqInt getWeight() const
+		{
+			return(m_weight);
+		}
+
+		TqBool needsCast() const
+		{
+			return(m_mustCast);
+		}
+
+	private:
+		SqFuncRef m_ref;
+		CqFuncDef* m_func;
+		TqInt m_weight;
+		TqBool m_mustCast;
+
+		friend bool operator<(const CqFunctionSignature& a, const CqFunctionSignature& b)
+		{
+			return(a.m_weight < b.m_weight);
+		}
+
+};
+
+
+TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     TqInt NewType = Type_Nil;
     TqInt cSpecifiedArgs = 0;
 
-    TqInt* aArgTypes = new TqInt[ m_aFuncRef.size() ];
-    std::vector<CqFuncDef*> aFuncs;
+	// This is the priority queue that we will build for the available 
+	// signatures. The top entry will be the most suitable for use.
+	std::priority_queue<CqFunctionSignature> functions;
 
     CqString strMyName = strName();
 
@@ -44,152 +93,169 @@ TqInt	CqParseNodeFunctionCall::TypeCheck( TqInt* pTypes, TqInt Count, TqBool Che
         pArg = pArg->pNext();
     }
 
-    TqUint i;
-    for ( i = 0; i < m_aFuncRef.size(); i++ )
+
+	// Build the queue
+    std::vector<SqFuncRef>::iterator i;
+    for ( i = m_aFuncRef.begin(); i != m_aFuncRef.end();  )
     {
-        aFuncs.push_back( CqFuncDef::GetFunctionPtr( m_aFuncRef[ i ] ) );
-        if ( aFuncs[ i ] != 0 )
+        CqFuncDef* pfunc = CqFuncDef::GetFunctionPtr( (*i) );
+        if( pfunc != 0 )
         {
-            TqInt cArgs = aFuncs[ i ] ->cTypeSpecLength();
+			TqInt weight = 0;
+			TqBool mustCast = TqFalse;
+            TqInt cArgs = pfunc->cTypeSpecLength();
             if ( ( cArgs > 0 ) &&
-                    ( ( !aFuncs[ i ] ->fVarying() && cArgs != cSpecifiedArgs ) ||
-                      ( aFuncs[ i ] ->fVarying() && cSpecifiedArgs < cArgs ) ) )
+                 ( ( !pfunc->fVarying() && cArgs != cSpecifiedArgs ) ||
+                   (  pfunc->fVarying() && cSpecifiedArgs < cArgs ) ) )
             {
-                m_aFuncRef.erase( m_aFuncRef.begin() + i );
-                aFuncs.erase( aFuncs.begin() + i );
-                i--;
+				// This function isn't suitable for the arguments list, as the
+				// argument counts don't match. Don't add it to the queue.
+				i = m_aFuncRef.erase(i);
+				continue;
             }
+			else
+			{
+				// Check the return type against the possible return types requested,
+				// a function that can return the right type without casting carries more weight.
+				TqInt castType, index;
+				if( (castType = FindCast(pfunc->Type(), pTypes, Count, index)) == Type_Nil )
+				{
+					// This signature cannot return any of the required types.
+					++i;
+					continue;
+				} 
+				else
+				{
+					// A function signature that can return one of the requested types without
+					// casting has more weight.
+					TqInt retType;
+					mustCast = TqTrue;
+					for(retType = 0; retType < Count; retType++)
+					{
+						if(castType == pfunc->Type())
+						{
+							weight++;
+							mustCast = TqFalse;
+							break;
+						}
+					}
+				}
+				// Check if the args list matches the arguments passed, the weight
+				// is adjusted according to the number of casts required.
+				pArg = m_pChild;
+				TqUint iArg = 0;
+				TqBool fArgsFailed = TqFalse;
+				while(pArg != 0)
+				{
+					CqParseNode * pNext = pArg->pNext();
+					
+					// Check if this is a varying length argument function.
+					if ( pfunc->cTypeSpecLength() <= iArg || ( pfunc->fVarying() && pfunc->cTypeSpecLength() == iArg ) )
+					{
+						// Not entirely happy about this, if the function takes a varying number
+						// of arguments, and this argument is amongst the varying list, then
+						// just accept it, no casting necessary.
+						break;
+					}
+
+					TqInt paramType = pfunc->aTypeSpec()[iArg];
+					TqInt castType;
+					TqBool castReturn = TqFalse;
+					if((castType = pArg->TypeCheck(&paramType, 1, castReturn, TqTrue)) == Type_Nil)
+					{
+						// We've found and argument that can't be cast to the suitable type
+						// so this signature is no good, don't add it.
+						fArgsFailed = TqTrue;
+						break;
+					}
+					else
+					{
+						// If we have an upper case type in the function declaration, then we need an actual variable.
+						if( (paramType & Type_Variable) && !pArg->IsVariableRef() )
+							break;
+						else
+							// Did the argument have to cast to match? If not, then increase the weight.
+							if(!castReturn)
+								weight++;
+					}
+
+					pArg = pNext;
+					iArg++;					
+				}
+
+				// If we got this far, the function signature matches, and we have a weight
+				// from the various data gathered, so add it to the queue.
+				if(!fArgsFailed)
+				{
+					functions.push(CqFunctionSignature((*i), pfunc, weight, mustCast));
+					++i;
+				}
+				else
+				{
+					// If the arguments check failed, this one can actually be removed from the list on the 
+					// node, as the arguments aren't going to change, only the potential return type
+					// for different calls to TypeCheck.
+					i = m_aFuncRef.erase(i);
+				}
+			}
         }
+		else
+			++i;
     }
 
-    // If we have eliminated all possibilities, then we have an error.
-    if ( m_aFuncRef.size() == 0 )
+	// If we don't have any candidate function signatures at all, then there is an error, so report
+	// it and retun nil.
+    if( functions.empty() )
     {
-        strErr += "Arguments to function not valid : ";
-        throw( strErr );
-        return ( Type_Nil );
+		if(!CheckOnly)
+		{
+			strErr += "Arguments to function not valid : ";
+			throw( strErr );
+		}
+		return ( Type_Nil );
     }
 
-    // Then typecheck the arguments.
-    pArg = m_pChild;
-    TqUint iArg = 0;
-    TqBool fVarLen = TqFalse;
-    while ( pArg != 0 )
-    {
-        CqParseNode * pNext = pArg->pNext();
-        // Build an array of possible types.
-        TqInt ctypes = 0;
-        for ( i = 0; i < m_aFuncRef.size(); i++ )
-        {
-            if ( aFuncs[ i ] != 0 )
-            {
-                if ( aFuncs[ i ] ->cTypeSpecLength() <= iArg || ( aFuncs[ i ] ->fVarying() && aFuncs[ i ] ->cTypeSpecLength() == iArg ) )
-                {
-                    if ( aFuncs[ i ] ->cTypeSpecLength() >= iArg && aFuncs[ i ] ->fVarying() )
-                        fVarLen = TqTrue;
-                    continue;
-                }
-                else
-                {
-                    aArgTypes[ i ] = aFuncs[ i ] ->aTypeSpec() [ iArg ];
-                    ctypes++;
-                }
-            }
-        }
+	const CqFunctionSignature& best = functions.top();
+	needsCast = best.needsCast();
 
-        // If we have no valid types to check, it must be a variable length function.
-        if ( ctypes == 0 )
-        {
-            pArg->TypeCheck( pAllTypes(), Type_Last - 1, TqTrue );
-            break;
-        }
-        // Now type check the argument
-        TqInt ArgType = pArg->TypeCheck( aArgTypes, ctypes, TqTrue );
-        // If no valid function exists for the argument, then this is an error.
-        if ( ArgType == Type_Nil && !fVarLen )
-        {
-            strErr += strErrDesc;
-            throw( strErr );
-            return ( Type_Nil );
-        }
-        else if ( ArgType != Type_Nil )
-            pArg->TypeCheck( &ArgType, 1 );	// Now do a real typecast
+	// If we are not just checking, at this point, we have the best function candidate,
+	// so cast the arguments if necessary, and apply a cast to the return if necessary.
+	if(!CheckOnly)
+	{
+		// Set the top function signature to the chosen one.
+		m_aFuncRef[0] = best.getRef();
 
-        // Check the list of possibles, and remove any that don't take the returned argument at the current point in the
-        // argument list.
-        for ( i = 0; i < m_aFuncRef.size(); i++ )
-        {
-            if ( strlen( aFuncs[ i ] ->strParams() ) <= iArg || aFuncs[ i ] ->strParams() [ iArg ] == '*' )
-                continue;
+		CqFuncDef* pfunc = best.getFunc();
+		// Apply casts to the arguments first.
+		pArg = m_pChild;
+		TqUint iArg = 0;
+		while ( pArg != 0 )
+		{
+			CqParseNode * pNext = pArg->pNext();
+			// Get the required type for the function signature.
+			TqInt argType;
+			if ( pfunc->cTypeSpecLength() <= iArg || ( pfunc->fVarying() && pfunc->cTypeSpecLength() == iArg ) )
+				break;
+			else
+				argType = pfunc->aTypeSpec() [ iArg ];
 
-            TqInt type = aFuncs[ i ] ->aTypeSpec() [ iArg ];
-            if ( ( type & Type_Mask ) != ( ArgType & Type_Mask ) )
-            {
-                m_aFuncRef.erase( m_aFuncRef.begin() + i );
-                aFuncs.erase( aFuncs.begin() + i );
-                i--;
-            }
-            else
-            {
-                // If we have an upper case type in the function declaration, then we need an actual variable.
-                if ( type & Type_Variable && !pArg->IsVariableRef() )
-                {
-                    strErrDesc = "Argument ";
-                    strErrDesc += static_cast<TqInt>( iArg );
-                    strErrDesc += " must be a variable";
-                    m_aFuncRef.erase( m_aFuncRef.begin() + i );
-                    aFuncs.erase( aFuncs.begin() + i );
-                    i--;
-                }
-            }
-        }
-        pArg = pNext;
-        iArg++;
-    }
+			// Now type check the argument, and cast if necessary
+			argType = pArg->TypeCheck( &argType, 1, needsCast, TqFalse );
 
+			pArg = pNext;
+			iArg++;
+		}
 
-    // Check if any of the remaining functions return the correct type.
-    for ( i = 0; i < m_aFuncRef.size(); i++ )
-    {
-        if ( aFuncs[ i ] != 0 )
-        {
-            TqInt j;
-            for ( j = 0; j < Count; j++ )
-            {
-                if ( aFuncs[ i ] ->Type() == pTypes[ j ] )
-                {
-                    // Set the selected function declaration as top.
-                    m_aFuncRef[ 0 ] = m_aFuncRef[ i ];
-                    return ( pTypes[ j ] );
-                }
-            }
-        }
-    }
-
-    // If we got here, we must cast the return value to a suitable type.
-    for ( i = 0; i < m_aFuncRef.size(); i++ )
-    {
-        if ( aFuncs[ i ] != 0 )
-        {
-            TqInt j;
-            for ( j = 0; j < Count; j++ )
-            {
-                if ( ( NewType = FindCast( aFuncs[ i ] ->Type(), pTypes, Count ) ) != Type_Nil )
-                {
-                    // Set the selected function declaration as top.
-                    m_aFuncRef[ 0 ] = m_aFuncRef[ i ];
-                    // Add a cast to the new type.
-                    CqParseNode* pCast = new CqParseNodeCast( NewType );
-                    LinkParent( pCast );
-                    return ( NewType );
-                }
-            }
-        }
-    }
-
-    strErr += strErrDesc;
-    throw( strErr );
-    return ( Type_Nil );
+		// Now check if the return value needs to be cast.
+		TqInt retType, index;
+		if ( ( retType = FindCast( pfunc->Type(), pTypes, Count, index ) ) != Type_Nil )
+		{
+			// Add a cast to the new type.
+			CqParseNode* pCast = new CqParseNodeCast( retType );
+			LinkParent( pCast );
+		}
+	}
+	return(best.getFunc()->Type());
 }
 
 
@@ -233,8 +299,9 @@ void CqParseNodeFunctionCall::CheckArgCast( std::vector<TqInt>& aRes )
         // Now check through to see if the specified arguments can be converted to the required ones.
         TqBool	fValid = TqTrue;
         TqUint j;
+		TqInt index;
         for ( j = 0; j < aTypes.size(); j++ )
-            if ( FindCast( aArgTypes[ j ], &aTypes[ j ], 1 ) == Type_Nil ) fValid = TqFalse;
+            if ( FindCast( aArgTypes[ j ], &aTypes[ j ], 1, index ) == Type_Nil ) fValid = TqFalse;
 
         // If we have found a match, return it.
         if ( !fValid )
@@ -268,7 +335,8 @@ void CqParseNodeFunctionCall::ArgCast( TqInt iIndex )
     {
         // Must get the next here incase the check inserts a cast.
         CqParseNode * pNext = pArg->pNext();
-        pArg->TypeCheck( &aTypes[ j++ ] );
+		TqBool needsCast;
+        pArg->TypeCheck( &aTypes[ j++ ], 1, needsCast, TqFalse );
         pArg = pNext;
     }
 }
@@ -280,7 +348,7 @@ void CqParseNodeFunctionCall::ArgCast( TqInt iIndex )
 /// to be, then later when the shader vm parses the .slx we can try and build
 /// a cast then, the same goes for the arguments.
 
-TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     TqInt NewType = Type_Nil;
 
@@ -292,7 +360,7 @@ TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count, TqBool C
     while (pArg != NULL)
     {
         CqParseNode *pNext = pArg->pNext();
-        pArg->TypeCheck( pAllTypes(), Type_Last - 1 );
+        pArg->TypeCheck( pAllTypes(), Type_Last - 1, needsCast, TqFalse );
 		strArgTypes+=CqParseNode::TypeIdentifier(pArg->ResType());
         pArg = pNext;
     };
@@ -331,7 +399,7 @@ TqInt	CqParseNodeUnresolvedCall::TypeCheck( TqInt* pTypes, TqInt Count, TqBool C
 /// CqParseNodeVariable::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Check if the variable type matches any of the requested ones.
     TqInt	MyType = ( ResType() & Type_Mask );
@@ -342,9 +410,14 @@ TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOn
             return ( MyType );
     }
     // If we got here, we are not of the correct type, so find a suitable cast.
-    TqInt NewType = FindCast( MyType, pTypes, Count );
-    CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
-    LinkParent( pCast );
+    TqInt index;
+	TqInt NewType = FindCast( MyType, pTypes, Count, index );
+	needsCast = TqTrue;
+	if(!CheckOnly)
+	{
+		CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
+		LinkParent( pCast );
+	}
 
     if ( NewType == Type_Nil && !CheckOnly )
     {
@@ -366,11 +439,12 @@ TqInt	CqParseNodeVariable::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOn
 /// CqParseNodeVariableArray::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeVariableArray::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeVariableArray::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Check if the child can be made into a float.
     TqInt aType = Type_Float;
-    if ( m_pChild && m_pChild->TypeCheck( &aType, 1, CheckOnly ) == Type_Nil )
+	TqBool temp;
+    if ( m_pChild && m_pChild->TypeCheck( &aType, 1, temp, CheckOnly ) == Type_Nil )
     {
         TqInt	IndexType = ( m_pChild->ResType() & Type_Mask );
         CqString strErr( strFileName() );
@@ -399,7 +473,7 @@ TqInt	CqParseNodeVariableArray::TypeCheck( TqInt* pTypes, TqInt Count, TqBool Ch
         return ( Type_Nil );
 	}
 
-    return ( CqParseNodeVariable::TypeCheck( pTypes, Count, CheckOnly ) );
+    return ( CqParseNodeVariable::TypeCheck( pTypes, Count, needsCast, CheckOnly ) );
 }
 
 
@@ -409,7 +483,7 @@ TqInt	CqParseNodeVariableArray::TypeCheck( TqInt* pTypes, TqInt Count, TqBool Ch
 
 extern EqShaderType gShaderType;
 
-TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // First check if the assign is to a read only variable.
     if( CqVarDef::GetVariablePtr( m_VarRef ) &&
@@ -432,7 +506,7 @@ TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
     TqInt	MyType = ( ResType() & Type_Mask );
     // Type check the assignment expression first.
     CqParseNode* pExpr = m_pChild;
-    if ( pExpr->TypeCheck( &MyType, 1, CheckOnly ) != MyType )
+    if ( pExpr->TypeCheck( &MyType, 1, needsCast, CheckOnly ) != MyType )
         return ( Type_Nil );	// TODO: Should throw an exception here.
 
     // Check if the variable type matches any of the requested ones.
@@ -443,9 +517,14 @@ TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
             return ( MyType );
     }
     // If we got here, we are not of the correct type, so find a suitable cast.
-    TqInt NewType = FindCast( MyType, pTypes, Count );
-    CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
-    LinkParent( pCast );
+    TqInt index;
+	TqInt NewType = FindCast( MyType, pTypes, Count, index );
+	needsCast = TqTrue;
+    if(!CheckOnly)
+	{
+		CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
+		LinkParent( pCast );
+	}
 
     if ( NewType == Type_Nil && !CheckOnly )
     {
@@ -467,11 +546,12 @@ TqInt	CqParseNodeAssign::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
 /// CqParseNodeAssignArray::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeAssignArray::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeAssignArray::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Check if the child can be made into a float.
     TqInt aType = Type_Float;
-    if ( m_pChild->pNext() ->TypeCheck( &aType, 1, CheckOnly ) == Type_Nil )
+	TqBool temp;
+    if ( m_pChild->pNext() ->TypeCheck( &aType, 1, temp, CheckOnly ) == Type_Nil )
     {
         TqInt	IndexType = ( m_pChild->pNext() ->ResType() & Type_Mask );
         CqString strErr( strFileName() );
@@ -484,7 +564,7 @@ TqInt	CqParseNodeAssignArray::TypeCheck( TqInt* pTypes, TqInt Count, TqBool Chec
         return ( Type_Nil );
     }
 
-    return ( CqParseNodeAssign::TypeCheck( pTypes, Count, CheckOnly ) );
+    return ( CqParseNodeAssign::TypeCheck( pTypes, Count, needsCast, CheckOnly ) );
 }
 
 
@@ -492,7 +572,7 @@ TqInt	CqParseNodeAssignArray::TypeCheck( TqInt* pTypes, TqInt Count, TqBool Chec
 /// CqParseNodeOp::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Check if both arguments to the operator match type.
     CqParseNode * pOperandA = m_pChild;
@@ -505,13 +585,17 @@ TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
     TqInt i;
     for ( i = 0; i < Count; i++ )
     {
-        if ( FindCast( TypeA, &pTypes[ i ], 1 ) != Type_Nil )
+        TqInt index;
+		if ( FindCast( TypeA, &pTypes[ i ], 1, index ) != Type_Nil )
         {
-            if ( FindCast( TypeB, &pTypes[ i ], 1 ) != Type_Nil )
+            if ( FindCast( TypeB, &pTypes[ i ], 1, index ) != Type_Nil )
             {
                 // Add any cast operators required.
-                pOperandA->TypeCheck( &pTypes[ i ], 1, CheckOnly );
-                pOperandB->TypeCheck( &pTypes[ i ], 1, CheckOnly );
+                if(!CheckOnly)
+				{
+					pOperandA->TypeCheck( &pTypes[ i ], 1, needsCast, CheckOnly );
+					pOperandB->TypeCheck( &pTypes[ i ], 1, needsCast, CheckOnly );
+				}
                 return ( pTypes[ i ] );
             }
         }
@@ -536,24 +620,29 @@ TqInt	CqParseNodeOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
 /// CqParseNodeRelOp::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     TqInt RelType;
-    RelType = CqParseNodeOp::TypeCheck( pAllTypes(), Type_Last - 1, CheckOnly );  
+    RelType = CqParseNodeOp::TypeCheck( pAllTypes(), Type_Last - 1, needsCast, CheckOnly );  
 
     if( RelType == Type_Nil)
     	return( RelType );
 
     // See if float is a requested type.
-    TqInt NewType;
-    if ( ( NewType = FindCast( Type_Float, pTypes, Count ) ) != Type_Nil )
+    TqInt NewType, index;
+    if ( ( NewType = FindCast( Type_Float, pTypes, Count, index ) ) != Type_Nil )
     {
-        if ( NewType == Type_Float ) return ( Type_Float );
+        if ( NewType == Type_Float ) 
+			return ( Type_Float );
         else
         {
-            CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
-            LinkParent( pCast );
-            return ( NewType );
+			needsCast = TqTrue;
+			if(!CheckOnly)
+			{
+				CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
+				LinkParent( pCast );
+			}
+			return ( NewType );
         }
     }
 
@@ -576,11 +665,11 @@ TqInt	CqParseNodeRelOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
 /// CqParseNodeUnaryOp::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeUnaryOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeUnaryOp::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Check if the operand can be cast to a requested type.
     if ( m_pChild != 0 )
-        return ( m_pChild->TypeCheck( pTypes, Count, CheckOnly ) );
+        return ( m_pChild->TypeCheck( pTypes, Count, needsCast, CheckOnly ) );
     else
         return ( Type_Nil );
 }
@@ -590,7 +679,7 @@ TqInt	CqParseNodeUnaryOp::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnl
 /// CqParseNodeOpDot::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     static TqInt aArgTypes[ 3 ] = {Type_Point, Type_Vector, Type_Normal};
 
@@ -605,20 +694,23 @@ TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckO
     // arguments can be made to match this, and always returns a float,
     // check this is valid.
 
-    TqInt RetType;
-    if ( ( RetType = FindCast( Type_Float, pTypes, Count ) ) != Type_Nil )
+    TqInt RetType, index;
+    if ( ( RetType = FindCast( Type_Float, pTypes, Count, index ) ) != Type_Nil )
     {
         TqBool fValid = TqFalse;
         TqUint i;
         for ( i = 0; i < sizeof( aArgTypes ) / sizeof( aArgTypes[ 0 ] ); i++ )
         {
-            if ( FindCast( TypeA, &aArgTypes[ i ], 1 ) != Type_Nil )
+            if ( FindCast( TypeA, &aArgTypes[ i ], 1, index ) != Type_Nil )
             {
-                if ( FindCast( TypeB, &aArgTypes[ i ], 1 ) != Type_Nil )
+                if ( FindCast( TypeB, &aArgTypes[ i ], 1, index ) != Type_Nil )
                 {
                     // Add any cast operators required.
-                    pOperandA->TypeCheck( &aArgTypes[ i ], 1, CheckOnly );
-                    pOperandB->TypeCheck( &aArgTypes[ i ], 1, CheckOnly );
+                    if(!CheckOnly)
+					{
+						pOperandA->TypeCheck( &aArgTypes[ i ], 1, needsCast, CheckOnly );
+						pOperandB->TypeCheck( &aArgTypes[ i ], 1, needsCast, CheckOnly );
+					}
                     fValid = TqTrue;
                     break;
                 }
@@ -626,10 +718,14 @@ TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckO
         }
         if ( fValid )
         {
-            if ( RetType != Type_Float )
+            if ( RetType != Type_Float)
             {
-                CqParseNodeCast * pCast = new CqParseNodeCast( RetType );
-                LinkParent( pCast );
+				needsCast = TqTrue;
+				if(!CheckOnly)
+				{
+					CqParseNodeCast * pCast = new CqParseNodeCast( RetType );
+					LinkParent( pCast );
+				}
             }
             return ( RetType );
         }
@@ -654,7 +750,7 @@ TqInt	CqParseNodeMathOpDot::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckO
 /// CqParseNodeConst::TypeCheck
 /// Perform a typecheck, and add cast if required.
 
-TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Check if the variable type matches any of the requested ones.
     TqInt	MyType = ResType();
@@ -665,13 +761,15 @@ TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
             return ( MyType );
     }
 
-    TqInt NewType = FindCast( MyType, pTypes, Count );
     // If we got here, we are not of the correct type, so find a suitable cast.
+    TqInt index;
+	TqInt NewType = FindCast( MyType, pTypes, Count, index );
+	needsCast = TqTrue;
     if ( !CheckOnly )
     {
         CqParseNodeCast * pCast = new CqParseNodeCast( NewType );
         LinkParent( pCast );
-    }
+	}
 
     if ( NewType == Type_Nil && !CheckOnly )
     {
@@ -693,11 +791,11 @@ TqInt CqParseNodeConst::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
 /// CqParseNodeCast::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Perform a typecheck on the cast expression
     CqParseNode * pExpr = m_pChild;
-    pExpr->TypeCheck( &m_tTo, 1, CheckOnly );
+    pExpr->TypeCheck( &m_tTo, 1, needsCast, CheckOnly );
 
     // Check the return type, and add another cast if necessary.
     // Note: We add another cast to allow for forced casts of function return types,
@@ -707,7 +805,8 @@ TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
     for ( i = 0; i < Count; i++ )
         if ( pTypes[ i ] == m_tTo ) return ( m_tTo );
 
-    TqInt NewType = FindCast( m_tTo, pTypes, Count );
+    TqInt index;
+	TqInt NewType = FindCast( m_tTo, pTypes, Count, index );
     if ( NewType == Type_Nil && !CheckOnly )
     {
         CqString strErr( strFileName() );
@@ -722,6 +821,7 @@ TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
     }
     else
     {
+		needsCast = TqTrue;
         if ( !CheckOnly )
         {
             CqParseNodeCast * pCast = new CqParseNodeCast( NewType );
@@ -737,14 +837,14 @@ TqInt	CqParseNodeCast::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
 /// CqParseNodeTriple::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     static TqInt ExprType = Type_Float;
     // Perform a typecheck on the three float expressions
     CqParseNode* pExpr = m_pChild;
     while ( pExpr != 0 )
     {
-        pExpr->TypeCheck( &ExprType, 1, CheckOnly );
+        pExpr->TypeCheck( &ExprType, 1, needsCast, CheckOnly );
         pExpr = pExpr->pNext();
     }
     // Check if expecting a triple, if not add a cast
@@ -753,9 +853,14 @@ TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
         if ( pTypes[ i ] == Type_Triple ) return ( Type_Triple );
 
     // If we got here, we are not of the correct type, so find a suitable cast.
-    TqInt NewType = FindCast( Type_Triple, pTypes, Count );
-    CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
-    LinkParent( pCast );
+	needsCast = TqTrue;
+    TqInt index;
+	TqInt NewType = FindCast( Type_Triple, pTypes, Count, index );
+    if(!CheckOnly)
+	{
+		CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
+		LinkParent( pCast );
+	}
 
     if ( NewType == Type_Nil && !CheckOnly )
     {
@@ -777,14 +882,14 @@ TqInt	CqParseNodeTriple::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
 /// CqParseNodeHexTuple::TypeCheck
 /// Do a type check based on suitable types requested, and add a cast in required.
 
-TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     static TqInt ExprType = Type_Float;
     // Perform a typecheck on the three float expressions
     CqParseNode* pExpr = m_pChild;
     while ( pExpr != 0 )
     {
-        pExpr->TypeCheck( &ExprType, 1, CheckOnly );
+        pExpr->TypeCheck( &ExprType, 1, needsCast, CheckOnly );
         pExpr = pExpr->pNext();
     }
     // Check if expecting a sixteentuple, if not add a cast
@@ -793,9 +898,14 @@ TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOn
         if ( pTypes[ i ] == Type_HexTuple ) return ( Type_HexTuple );
 
     // If we got here, we are not of the correct type, so find a suitable cast.
-    TqInt NewType = FindCast( Type_Matrix, pTypes, Count );
-    CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
-    LinkParent( pCast );
+	needsCast = TqTrue;
+    TqInt index;
+	TqInt NewType = FindCast( Type_Matrix, pTypes, Count, index );
+    if(!CheckOnly)
+	{
+		CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
+		LinkParent( pCast );
+	}
 
     if ( NewType == Type_Nil && !CheckOnly )
     {
@@ -816,7 +926,7 @@ TqInt	CqParseNodeHexTuple::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOn
 ///---------------------------------------------------------------------
 /// CqParseNodeCommFunction::TypeCheck
 
-TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Check if the variable type matches any of the requested ones.
     TqInt	MyType = ResType();
@@ -827,9 +937,14 @@ TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count, TqBool Che
             return ( MyType );
     }
     // If we got here, we are not of the correct type, so find a suitable cast.
-    TqInt NewType = FindCast( MyType, pTypes, Count );
-    CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
-    LinkParent( pCast );
+	needsCast = TqTrue;
+    TqInt index;
+	TqInt NewType = FindCast( MyType, pTypes, Count, index );
+    if(!CheckOnly)
+	{
+		CqParseNodeCast* pCast = new CqParseNodeCast( NewType );
+		LinkParent( pCast );
+	}
 
     if ( NewType == Type_Nil && !CheckOnly )
     {
@@ -851,12 +966,12 @@ TqInt CqParseNodeCommFunction::TypeCheck( TqInt* pTypes, TqInt Count, TqBool Che
 /// CqParseNodeQCond::TypeCheck
 /// Type check the return from a conditional statement and cast the true/false expressions if necessary.
 
-TqInt	CqParseNodeQCond::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly )
+TqInt	CqParseNodeQCond::TypeCheck( TqInt* pTypes, TqInt Count,  TqBool& needsCast, TqBool CheckOnly )
 {
     // Ensure that the conditional expression is type checked.
     CqParseNode * pCondExp = m_pChild;
     assert( pCondExp != 0 );
-    pCondExp->TypeCheck( pAllTypes(), Type_Last - 1, CheckOnly );
+    pCondExp->TypeCheck( pAllTypes(), Type_Last - 1, needsCast, CheckOnly );
 
     // Check if both expressions match type.
     CqParseNode * pTrue = m_pChild->pNext();
@@ -871,13 +986,17 @@ TqInt	CqParseNodeQCond::TypeCheck( TqInt* pTypes, TqInt Count, TqBool CheckOnly
     TqInt i;
     for ( i = 0; i < Count; i++ )
     {
-        if ( FindCast( TypeT, &pTypes[ i ], 1 ) != Type_Nil )
+		TqInt index;
+        if ( FindCast( TypeT, &pTypes[ i ], 1, index ) != Type_Nil )
         {
-            if ( FindCast( TypeF, &pTypes[ i ], 1 ) != Type_Nil )
+            if ( FindCast( TypeF, &pTypes[ i ], 1, index ) != Type_Nil )
             {
                 // Add any cast operators required.
-                pTrue->TypeCheck( &pTypes[ i ], 1, CheckOnly );
-                pFalse->TypeCheck( &pTypes[ i ], 1, CheckOnly );
+                if(!CheckOnly)
+				{
+					pTrue->TypeCheck( &pTypes[ i ], 1, needsCast, CheckOnly );
+					pFalse->TypeCheck( &pTypes[ i ], 1, needsCast, CheckOnly );
+				}
                 return ( pTypes[ i ] );
             }
         }
