@@ -23,6 +23,8 @@
 		\author Paul C. Gregory (pgregory@aqsis.com)
 */
 
+#include	"MultiTimer.h"
+
 #include	"aqsis.h"
 
 #ifdef WIN32
@@ -37,6 +39,7 @@
 #include	"imagers.h"
 
 #include	<algorithm>
+#include	<valarray>
 
 
 
@@ -65,6 +68,8 @@ TqInt	CqBucket::m_NumDofBounds;
 CqImageBuffer* CqBucket::m_ImageBuffer;
 std::vector<CqBound>		CqBucket::m_DofBounds;
 std::vector<CqImagePixel>	CqBucket::m_aieImage;
+std::vector<SqSampleData>	CqBucket::m_SamplePoints;
+TqInt	CqBucket::m_NextSamplePoint = 0;
 std::vector<std::vector<CqVector2D> >	CqBucket::m_aSamplePositions;
 std::vector<TqFloat> CqBucket::m_aFilterValues;
 std::vector<TqFloat> CqBucket::m_aDatas;
@@ -76,7 +81,7 @@ std::vector<TqFloat> CqBucket::m_aCoverages;
  *  Clear,Allocate, Init. the m_aieImage samples
  */
 
-void CqBucket::InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqInt ysize, TqBool fJitter, TqBool empty )
+void CqBucket::PrepareBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqInt ysize, TqBool fJitter, TqBool empty )
 {
     m_XOrigin = xorigin;
     m_YOrigin = yorigin;
@@ -96,8 +101,12 @@ void CqBucket::InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqIn
     // Allocate the image element storage if this is the first bucket
     if(m_aieImage.empty())
     {
+		SqImageSample::SetSampleSize(QGetRenderContext() ->GetOutputDataTotalSize());
+
         m_aieImage.resize( m_RealWidth * m_RealHeight );
 		m_aSamplePositions.resize( m_RealWidth * m_RealHeight );
+		m_SamplePoints.resize( m_RealWidth * m_RealHeight * m_PixelXSamples * m_PixelYSamples );
+		m_NextSamplePoint = 0;
 
 		CalculateDofBounds();
 
@@ -109,18 +118,18 @@ void CqBucket::InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqIn
             {
                 m_aieImage[which].Clear();
                 m_aieImage[which].AllocateSamples( m_PixelXSamples, m_PixelYSamples );
-                m_aieImage[which].InitialiseSamples( m_aSamplePositions[which], fJitter );
-				m_aieImage[which].ShuffleSamples();
+                m_aieImage[which].InitialiseSamples( m_aSamplePositions[which] );
+				if(fJitter)
+					m_aieImage[which].JitterSamples(m_aSamplePositions[which]);
 
                 which++;
             }
         }
     }
 
-	//:TODO: AGG, sort out sample shuffling.
-	// now shuffle the pixels around and add in the pixel offset to the position.
 	std::random_shuffle(m_aieImage.begin(), m_aieImage.end());
 
+	// Jitter the samplepoints and adjust them for the new bucket position.
 	TqInt which = 0;
 	TqInt numPixels = m_RealWidth*m_RealHeight;
 	for ( TqInt i = 0; i < m_RealHeight; i++ )
@@ -133,6 +142,8 @@ void CqBucket::InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqIn
 			if(!empty)
 				m_aieImage[which].Clear();
 
+			//if(fJitter)
+			//	m_aieImage[which].JitterSamples(m_aSamplePositions[which]);
 			m_aieImage[which].OffsetSamples( bPos2, m_aSamplePositions[which] );
 
 			which++;
@@ -284,11 +295,11 @@ void CqBucket::InitialiseFilterValues()
 /** Combine the subsamples into single pixel samples and coverage information.
  */
 
-void CqBucket::CombineElements()
+void CqBucket::CombineElements(enum EqFilterDepth filterdepth, CqColor zThreshold)
 {
     std::vector<CqImagePixel>::iterator end = m_aieImage.end();
     for ( std::vector<CqImagePixel>::iterator i = m_aieImage.begin(); i != end ; i++ )
-        i->Combine();
+		i->Combine(filterdepth, zThreshold);
 }
 
 
@@ -363,24 +374,6 @@ TqFloat CqBucket::Depth( TqInt iXPos, TqInt iYPos )
 }
 
 
-//----------------------------------------------------------------------
-/** Get the maximum sample depth for the specified screen position.
- * If position is outside bucket, returns FLT_MAX.
- * \param iXPos Screen position of sample.
- * \param iYPos Screen position of sample.
- */
-
-TqFloat CqBucket::MaxDepth( TqInt iXPos, TqInt iYPos )
-{
-    CqImagePixel * pie;
-    ImageElement( iXPos, iYPos, pie );
-    if( NULL != pie )
-        return ( pie->MaxDepth() );
-    else
-        return ( FLT_MAX );
-}
-
-
 //----------------------------------------------------------------------
 /** Get a pointer to the samples for a given pixel.
  * If position is outside bucket, returns NULL.
@@ -398,23 +391,6 @@ const TqFloat* CqBucket::Data( TqInt iXPos, TqInt iYPos )
         return ( NULL );
 }
 
-//----------------------------------------------------------------------
-/** Get count of samples.
- * If position is outside bucket, returns 0.
- * \param iXPos Screen position of sample.
- * \param iYPos Screen position of sample.
- */
-
-TqInt CqBucket::DataSize( TqInt iXPos, TqInt iYPos )
-{
-    CqImagePixel * pie;
-    ImageElement( iXPos, iYPos, pie );
-    if( NULL != pie )
-        return ( pie->DataSize() );
-    else
-        return ( 0 );
-}
-
 
 //----------------------------------------------------------------------
 /** Filter the samples in this bucket according to type and filter widths.
@@ -511,7 +487,7 @@ void CqBucket::FilterBucket(TqBool empty)
 									{
 										SqImageSample& pSample = pie2->OpaqueValues( sampleIndex );
 										for ( TqInt k = 0; k < datasize; ++k )
-											samples[k] += pSample.m_Data[k] * g;
+											samples[k] += pSample.Data()[k] * g;
 										sampleCounts[pixelIndex]++;
 									}
 								}
@@ -646,7 +622,7 @@ void CqBucket::FilterBucket(TqBool empty)
 										{
 											SqImageSample& pSample = pie2->OpaqueValues( sampleIndex );
 											for ( TqInt k = 0; k < datasize; ++k )
-												samples[k] += pSample.m_Data[k] * g;
+												samples[k] += pSample.Data()[k] * g;
 											SampleCount++;
 										}
 									}
@@ -709,12 +685,13 @@ void CqBucket::FilterBucket(TqBool empty)
         {
             SqImageSample& spl = pie2->GetPixelSample();
 			for (TqInt k=0; k < datasize; k++)
-                spl.m_Data[k] = m_aDatas[ i * datasize + k ];
-            spl.SetCoverage( m_aCoverages[ i++ ] );
+                spl.Data()[k] = m_aDatas[ i * datasize + k ];
+            TqFloat* sample_data = spl.Data();
+			sample_data[Sample_Coverage] = m_aCoverages[ i++ ];
 	
 			// Calculate the alpha as the combination of the opacity and the coverage.
-			TqFloat a = ( spl.Os()[0] + spl.Os()[1] + spl.Os()[2] ) / 3.0f;
-			pie2->SetAlpha(a * spl.Coverage());
+			TqFloat a = ( sample_data[Sample_ORed] + sample_data[Sample_OGreen] + sample_data[Sample_OBlue] ) / 3.0f;
+			pie2->SetAlpha(a * sample_data[Sample_Coverage]);
 
             pie2++;
         }
@@ -726,10 +703,10 @@ void CqBucket::FilterBucket(TqBool empty)
 
     if ( NULL != QGetRenderContext() ->optCurrent().pshadImager() && QGetRenderContext() ->optCurrent().pshadImager() ->pShader() )
     {
-        QGetRenderContext() ->Stats().MakeFilterBucket().Stop();
         // Init & Execute the imager shader
 
         QGetRenderContext() ->optCurrent().InitialiseColorImager( this );
+		TIME_SCOPE("Imager shading")
 
         if ( fImager )
         {
@@ -755,7 +732,6 @@ void CqBucket::FilterBucket(TqBool empty)
                 pie += xlen;
             }
         }
-        QGetRenderContext() ->Stats().MakeFilterBucket().Start();
     }
 }
 
@@ -934,9 +910,9 @@ void CqBucket::QuantizeBucket()
                     for( sampleindex = startindex; sampleindex < endindex; sampleindex++ )
                     {
                         double d;
-                        if ( modf( one * pie2->GetPixelSample().m_Data[sampleindex] + ditheramplitude * random.RandomFloat(), &d ) > 0.5 ) d += 1.0f;
+                        if ( modf( one * pie2->GetPixelSample().Data()[sampleindex] + ditheramplitude * random.RandomFloat(), &d ) > 0.5 ) d += 1.0f;
                         d = CLAMP( d, min, max );
-                        pie2->GetPixelSample().m_Data[sampleindex] = d;
+                        pie2->GetPixelSample().Data()[sampleindex] = d;
                     }
                     pie2++;
                 }
@@ -959,6 +935,7 @@ void CqBucket::ShutdownBucket()
 	for( i=m_aSamplePositions.begin(); i!=m_aSamplePositions.end(); i++ )
 		(*i).clear();
 	m_aSamplePositions.clear();
+	m_SamplePoints.clear();
 }
 
 
