@@ -113,11 +113,9 @@ public:
     virtual	CqColor Opacity( TqInt iXPos, TqInt iYPos );
     virtual	TqFloat Coverage( TqInt iXPos, TqInt iYPos );
     virtual	TqFloat Depth( TqInt iXPos, TqInt iYPos );
-    virtual	TqFloat MaxDepth( TqInt iXPos, TqInt iYPos );
-    virtual	TqInt DataSize( TqInt iXPos, TqInt iYPos );
     virtual	const TqFloat* Data( TqInt iXPos, TqInt iYPos );
 
-    static	void	InitialiseBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqInt ysize, TqBool fJitter = TqTrue, TqBool empty = TqFalse );
+    static	void	PrepareBucket( TqInt xorigin, TqInt yorigin, TqInt xsize, TqInt ysize, TqBool fJitter = TqTrue, TqBool empty = TqFalse );
  	static	void	CalculateDofBounds();
     static	void	InitialiseFilterValues();
     static	void	ImageElement( TqInt iXPos, TqInt iYPos, CqImagePixel*& pie )
@@ -132,8 +130,25 @@ public:
 		TqInt i = ( ( iYPos + m_DiscreteShiftY ) * ( m_RealWidth ) ) + ( iXPos + m_DiscreteShiftX );
 		pie = &m_aieImage[ i ];
 	}
+	static CqImagePixel& ImageElement(TqInt index)
+	{
+		assert(index < m_aieImage.size());
+		return(m_aieImage[index]);
+	}
 
-    static	void	CombineElements();
+	static	std::vector<SqSampleData>& SamplePoints()
+	{
+		return(m_SamplePoints);
+	}
+
+	static TqInt GetNextSamplePointIndex()
+	{
+		TqInt index = m_NextSamplePoint;
+		m_NextSamplePoint++;
+		return(index);
+	}
+
+    static	void	CombineElements(enum EqFilterDepth eDepthFilter, CqColor zThreshold);
     void	FilterBucket(TqBool empty);
     void	ExposeBucket();
     void	QuantizeBucket();
@@ -246,6 +261,8 @@ private:
 	static	TqInt	m_NumDofBounds;
 	static	std::vector<CqBound>		m_DofBounds;
     static	std::vector<CqImagePixel>	m_aieImage;
+	static	std::vector<SqSampleData>	m_SamplePoints;
+	static	TqInt	m_NextSamplePoint;
     static	std::vector<std::vector<CqVector2D> >	m_aSamplePositions;///< Vector of vectors of jittered sample positions precalculated.
     static	std::vector<TqFloat>	m_aFilterValues;				///< Vector of filter weights precalculated.
     static	std::vector<TqFloat>	m_aDatas;
