@@ -141,7 +141,7 @@ TqInt CqCSGTreeNode::cChildren()
  *
  *	@param	samples	Array of samples to pass through the CSG tree.
  */
-void CqCSGTreeNode::ProcessTree( std::vector<SqImageSample>& samples )
+void CqCSGTreeNode::ProcessTree( std::list<SqImageSample>& samples )
 {
     // Follow the tree back up to the top, then process the list from there
     boost::shared_ptr<CqCSGTreeNode> pTop = shared_from_this();
@@ -163,7 +163,7 @@ void CqCSGTreeNode::ProcessTree( std::vector<SqImageSample>& samples )
  *
  *	@param	samples	Array of samples to process.
  */
-void CqCSGTreeNode::ProcessSampleList( std::vector<SqImageSample>& samples )
+void CqCSGTreeNode::ProcessSampleList( std::list<SqImageSample>& samples )
 {
     // First process any children nodes.
     // Process all nodes depth first.
@@ -189,7 +189,7 @@ void CqCSGTreeNode::ProcessSampleList( std::vector<SqImageSample>& samples )
 
     // Find out if the camera is starting inside a solid. This is the case if you
     // see an odd number of walls for that solid when looking out.
-    std::vector<SqImageSample>::iterator i;
+    std::list<SqImageSample>::iterator i;
     TqInt j = 0;
     for ( i = samples.begin(); i != samples.end(); ++i, ++j )
     {
@@ -249,10 +249,10 @@ void CqCSGTreeNode::ProcessSampleList( std::vector<SqImageSample>& samples )
  *
  *	@param	samples	Array of samples to process.
  */
-void CqCSGNodePrimitive::ProcessSampleList( std::vector<SqImageSample>& samples )
+void CqCSGNodePrimitive::ProcessSampleList( std::list<SqImageSample>& samples )
 {
     // Now go through samples, clearing samples related to this node.
-    std::vector<SqImageSample>::iterator i;
+    std::list<SqImageSample>::iterator i;
     for ( i = samples.begin(); i != samples.end(); ++i )
     {
         if ( i->m_pCSGNode.get() == this )
