@@ -23,6 +23,8 @@
 		\author Paul C. Gregory (pgregory@aqsis.com)
 */
 
+#include	"MultiTimer.h"
+
 #include	"aqsis.h"
 
 #ifdef WIN32
@@ -146,8 +148,8 @@ void	CqImageBuffer::SetImage()
     m_CropWindowXMax = static_cast<TqInt>( CLAMP( CEIL( m_iXRes * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 1 ] ), 0, m_iXRes ) );
     m_CropWindowYMin = static_cast<TqInt>( CLAMP( CEIL( m_iYRes * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 2 ] ), 0, m_iYRes ) );
     m_CropWindowYMax = static_cast<TqInt>( CLAMP( CEIL( m_iYRes * QGetRenderContext() ->optCurrent().GetFloatOption( "System", "CropWindow" ) [ 3 ] ), 0, m_iYRes ) );
-    m_cXBuckets = ( m_iXRes / m_XBucketSize ) + 1;
-    m_cYBuckets = ( m_iYRes / m_YBucketSize ) + 1;
+    m_cXBuckets = ( ( m_iXRes + ( m_XBucketSize-1 ) ) / m_XBucketSize );
+    m_cYBuckets = ( ( m_iYRes + ( m_YBucketSize-1 ) ) / m_YBucketSize );
     m_PixelXSamples = QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "PixelSamples" ) [ 0 ];
     m_PixelYSamples = QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "PixelSamples" ) [ 1 ];
 
@@ -155,7 +157,12 @@ void	CqImageBuffer::SetImage()
     m_ClippingFar = static_cast<TqFloat>( QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 1 ] );
     m_DisplayMode = QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "DisplayMode" ) [ 0 ];
 
-	CqBucket::SetImageBuffer(this);
+    m_MaxEyeSplits = 10;
+    const TqInt* poptEyeSplits = QGetRenderContext() ->optCurrent().GetIntegerOption( "limits", "eyesplits" );
+    if ( poptEyeSplits != 0 )
+        m_MaxEyeSplits = poptEyeSplits[ 0 ];
+
+    CqBucket::SetImageBuffer(this);
     m_Buckets.resize( m_cYBuckets );
     std::vector<std::vector<CqBucket> >::iterator i;
     for( i = m_Buckets.begin(); i!=m_Buckets.end(); i++)
@@ -213,26 +220,27 @@ TqBool CqImageBuffer::CullSurface( CqBound& Bound, const boost::shared_ptr<CqBas
 {
     // If the primitive is completely outside of the hither-yon z range, cull it.
     if ( Bound.vecMin().z() >= QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 1 ] ||
-            Bound.vecMax().z() <= QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 0 ] )
+         Bound.vecMax().z() <= QGetRenderContext() ->optCurrent().GetFloatOption( "System", "Clipping" ) [ 0 ] )
         return ( TqTrue );
 
+	if(QGetRenderContext()->clippingVolume().whereIs(Bound) == CqBound::Side_Outside)
+	{
+		return(TqTrue);
+	}
+
     // If the primitive spans the epsilon plane and the hither plane and can be split,
     if ( Bound.vecMin().z() <= 0.0f && Bound.vecMax().z() > FLT_EPSILON )
     {
         // Mark the primitive as not dicable.
         pSurface->ForceUndiceable();
-        TqInt MaxEyeSplits = 10;
-        const TqInt* poptEyeSplits = QGetRenderContext() ->optCurrent().GetIntegerOption( "limits", "eyesplits" );
-        if ( poptEyeSplits != 0 )
-            MaxEyeSplits = poptEyeSplits[ 0 ];
 
-        if ( pSurface->EyeSplitCount() > MaxEyeSplits )
+        if ( pSurface->EyeSplitCount() > m_MaxEyeSplits )
         {
             CqString objname( "unnamed" );
             const CqString* pattrName = pSurface->pAttributes() ->GetStringAttribute( "identifier", "name" );
             if ( pattrName != 0 ) objname = pattrName[ 0 ];
 			std::cerr << warning << "Max eyesplits for object \"" << objname.c_str() << "\" exceeded" << std::endl;
-			return( TqTrue );
+		    return( TqTrue );
         }
         return ( TqFalse );
     }
@@ -406,7 +414,7 @@ TqBool CqImageBuffer::OcclusionCullSurface( const boost::shared_ptr<CqBasicSurfa
         CqString objname( "unnamed" );
         const CqString* pattrName = pSurface->pAttributes() ->GetStringAttribute( "identifier", "name" );
         if( pattrName )	objname = *pattrName;
-        std::cerr << info << "GPrim: \"" << objname << "\" occlusion culled" << std::endl;
+            std::cerr << info << "GPrim: \"" << objname << "\" occlusion culled" << std::endl;
         STATS_INC( GPR_culled );
         return TqTrue;
     }
@@ -511,7 +519,7 @@ TqBool CqImageBuffer::PushMPGForward( CqMicroPolygon* pmpg, TqInt Col, TqInt Row
     BucketMin -= FilterWidth;
     BucketMax += FilterWidth;
 
-    CqBound	B( pmpg->GetTotalBound() );
+    const CqBound&	B = pmpg->GetTotalBound();
 
     const CqVector3D& vMin = B.vecMin();
     const CqVector3D& vMax = B.vecMax();
@@ -566,7 +574,7 @@ TqBool CqImageBuffer::PushMPGDown( CqMicroPolygon* pmpg, TqInt Col, TqInt Row )
     BucketMin -= FilterWidth;
     BucketMax += FilterWidth;
 
-    CqBound	B( pmpg->GetTotalBound( ) );
+    const CqBound&	B = pmpg->GetTotalBound( );
 
     const CqVector3D& vMin = B.vecMin();
     const CqVector3D& vMax = B.vecMax();
@@ -693,12 +701,12 @@ void CqImageBuffer::RenderMPGs( long xmin, long xmax, long ymin, long ymax )
 
 void CqImageBuffer::RenderMicroPoly( CqMicroPolygon* pMPG, long xmin, long xmax, long ymin, long ymax )
 {
-	CqBound Bound = pMPG->GetTotalBound();
+	const CqBound& Bound = pMPG->GetTotalBound();
 
 	// if bounding box is outside our viewing range, then cull it.
 	if ( Bound.vecMax().x() < xmin || Bound.vecMax().y() < ymin ||
-		Bound.vecMin().x() > xmax || Bound.vecMin().y() > ymax ||
-		Bound.vecMin().z() > ClippingFar() || Bound.vecMax().z() < ClippingNear())
+		 Bound.vecMin().x() > xmax || Bound.vecMin().y() > ymax ||
+		 Bound.vecMin().z() > ClippingFar() || Bound.vecMax().z() < ClippingNear())
 	{
 		STATS_INC( MPG_culled );
 		return;
@@ -708,7 +716,7 @@ void CqImageBuffer::RenderMicroPoly( CqMicroPolygon* pMPG, long xmin, long xmax,
     // Must check if colour is needed, as if not, the variable will have been deleted from the grid.
     if ( QGetRenderContext() ->pDDmanager() ->fDisplayNeeds( "Ci" ) )
     {
-        m_CurrentMpgSampleInfo.m_Colour = pMPG->colColor();
+        m_CurrentMpgSampleInfo.m_Colour = pMPG->colColor()[0];
     }
     else
     {
@@ -718,8 +726,8 @@ void CqImageBuffer::RenderMicroPoly( CqMicroPolygon* pMPG, long xmin, long xmax,
     // Must check if opacity is needed, as if not, the variable will have been deleted from the grid.
     if ( QGetRenderContext() ->pDDmanager() ->fDisplayNeeds( "Oi" ) )
     {
-        m_CurrentMpgSampleInfo.m_Opacity = pMPG->colOpacity();
-        m_CurrentMpgSampleInfo.m_Occludes = pMPG->colOpacity() >= gColWhite;
+        m_CurrentMpgSampleInfo.m_Opacity = pMPG->colOpacity()[0];
+        m_CurrentMpgSampleInfo.m_Occludes = m_CurrentMpgSampleInfo.m_Opacity >= gColWhite;
     }
     else
     {
@@ -748,6 +756,7 @@ void CqImageBuffer::RenderMicroPoly( CqMicroPolygon* pMPG, long xmin, long xmax,
 	}
 }
 
+
 // this function assumes that either dof or mb or both are being used.
 void CqImageBuffer::RenderMPG_MBOrDof( CqMicroPolygon* pMPG,
 										long xmin, long xmax, long ymin, long ymax,
@@ -764,6 +773,8 @@ void CqImageBuffer::RenderMPG_MBOrDof( CqMicroPolygon* pMPG,
 
 	CqHitTestCache hitTestCache;
 	TqBool cachedHitData = TqFalse;
+	pMPG->CacheHitTestValues(&hitTestCache);
+	cachedHitData = TqTrue;
 
 	TqBool mustDraw = !m_CurrentGridInfo.m_IsCullable;
 
@@ -784,8 +795,8 @@ void CqImageBuffer::RenderMPG_MBOrDof( CqMicroPolygon* pMPG,
 	TqInt currentIndex = 0;
     for ( TqInt bound_numMB = 0; bound_numMB < bound_maxMB; bound_numMB++ )
     {
-        TqFloat time0;
-        TqFloat time1;
+        TqFloat time0 = m_CurrentGridInfo.m_ShutterOpenTime;
+        TqFloat time1 = m_CurrentGridInfo.m_ShutterCloseTime;
         const CqBound& Bound = pMPG->SubBound( bound_numMB, time0 );
 
 		// get the index of the first and last samples that can fall inside
@@ -873,378 +884,36 @@ void CqImageBuffer::RenderMPG_MBOrDof( CqMicroPolygon* pMPG,
 				continue;
 			}
 
-			// Now go across all pixels touched by the micropolygon bound.
-			// The first pixel position is at (sX, sY), the last one
-			// at (eX, eY).
-			TqInt eX = CEIL( bmaxx );
-			TqInt eY = CEIL( bmaxy );
-			if ( eX > xmax ) eX = xmax;
-			if ( eY > ymax ) eY = ymax;
-
-			TqInt sX = FLOOR( bminx );
-			TqInt sY = FLOOR( bminy );
-			if ( sY < ymin ) sY = ymin;
-			if ( sX < xmin ) sX = xmin;
-
-			CqImagePixel* pie, *pie2;
-
-			TqInt nextx = Bucket.RealWidth();
-			Bucket.ImageElement( sX, sY, pie );
-
-			for( int iY = sY; iY < eY; ++iY)
+			if(UsingDof)
 			{
-				pie2 = pie;
-				pie += nextx;
-
-				for(int iX = sX; iX < eX; ++iX, ++pie2)
-				{
-					// only bother sampling if the mpg is not occluded in this pixel.
-					if(mustDraw || bminz <= pie2->MaxDepth())
-					{
-						TqInt index;
-						if(UsingDof)
-						{
-							// when using dof only one sample per pixel can
-							// possibbly hit (the one corresponding to the
-							// current bounding box).
-							index = pie2->GetDofOffsetIndex(bound_numDof);
-						}
-						else
-						{
-							// when using mb without dof, a range of samples
-							// may have times within the current mb bounding box.
-							index = indexT0;
-						}
-
-						// loop over potential samples
-						do
-						{
-							const SqSampleData& sampleData = pie2->SampleData( index );
-							const CqVector2D& vecP = sampleData.m_Position;
-							const TqFloat time = sampleData.m_Time;
-
-							index++;
-
-							CqStats::IncI( CqStats::SPL_count );
-
-							if(IsMoving && (time < time0 || time > time1))
-							{
-								continue;
-							}
-
-							// check if sample lies inside mpg bounding box.
-							if ( UsingDof )
-							{
-								CqBound DofBound(bminx, bminy, bminz, bmaxx, bmaxy, bmaxz);
-
-								if(!DofBound.Contains2D( vecP ))
-									continue;
-
-								// Check to see if the sample is within the sample's level of detail
-								if ( UsingLevelOfDetail)
-								{
-									TqFloat LevelOfDetail = sampleData.m_DetailLevel;
-									if ( LodBounds[ 0 ] > LevelOfDetail || LevelOfDetail >= LodBounds[ 1 ] )
-									{
-										continue;
-									}
-								}
-
-
-								CqStats::IncI( CqStats::SPL_bound_hits );
-
-								// Now check if the subsample hits the micropoly
-								TqBool SampleHit;
-								TqFloat D;
-
-								SampleHit = pMPG->Sample( sampleData, D, time, UsingDof );
-								if ( SampleHit )
-								{
-									sample_hits++;
-									// note index has already been incremented, so we use the previous value.
-									StoreSample( pMPG, pie2, index-1, D );
-								}
-							}
-							else
-							{
-								if(!Bound.Contains2D( vecP ))
-									continue;
-								// Check to see if the sample is within the sample's level of detail
-								if ( UsingLevelOfDetail)
-								{
-									TqFloat LevelOfDetail = sampleData.m_DetailLevel;
-									if ( LodBounds[ 0 ] > LevelOfDetail || LevelOfDetail >= LodBounds[ 1 ] )
-									{
-										continue;
-									}
-								}
-
-
-								CqStats::IncI( CqStats::SPL_bound_hits );
-
-								// Now check if the subsample hits the micropoly
-								TqBool SampleHit;
-								TqFloat D;
-
-								pMPG->CacheHitTestValues(&hitTestCache);
-								cachedHitData = TqTrue;
-
-								SampleHit = pMPG->Sample( sampleData, D, time );
-								if ( SampleHit )
-								{
-									sample_hits++;
-									// note index has already been incremented, so we use the previous value.
-									StoreSample( pMPG, pie2, index-1, D );
-								}
-							}
-						} while (!UsingDof && index < indexT1);
-					}
-				}
+				CqBound DofBound(bminx, bminy, bminz, bmaxx, bmaxy, bmaxz);
+				CqOcclusionBox::KDTree()->SampleMPG(pMPG, DofBound, IsMoving, time0, time1, TqTrue, bound_numDof, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
+			}
+			else
+			{
+				CqOcclusionBox::KDTree()->SampleMPG(pMPG, Bound, IsMoving, time0, time1, TqFalse, 0, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
 			}
 		}
     }
 }
 
+
+
 // this function assumes that neither dof or mb are being used. it is much
 // simpler than the general case dealt with above.
 void CqImageBuffer::RenderMPG_Static( CqMicroPolygon* pMPG, long xmin, long xmax, long ymin, long ymax )
 {
-    CqBucket & Bucket = CurrentBucket();
-    CqStats& theStats = QGetRenderContext() ->Stats();
-
-    const TqFloat* LodBounds = m_CurrentGridInfo.m_LodBounds;
-    TqBool UsingLevelOfDetail = LodBounds[ 0 ] >= 0.0f;
-
-    TqInt sample_hits = 0;
-    TqFloat shd_rate = m_CurrentGridInfo.m_ShadingRate;
-
 	CqHitTestCache hitTestCache;
-	TqBool cachedHitData = TqFalse;
+	pMPG->CacheHitTestValues(&hitTestCache);
 
-	TqBool mustDraw = !m_CurrentGridInfo.m_IsCullable;
+    const CqBound& Bound = pMPG->GetTotalBound();
 
-    CqBound Bound = pMPG->GetTotalBound();
-
-	TqFloat bminx = Bound.vecMin().x();
-	TqFloat bmaxx = Bound.vecMax().x();
-	TqFloat bminy = Bound.vecMin().y();
-	TqFloat bmaxy = Bound.vecMax().y();
-	TqFloat bminz = Bound.vecMin().z();
-
-	// Now go across all pixels touched by the micropolygon bound.
-	// The first pixel position is at (sX, sY), the last one
-	// at (eX, eY).
-	TqInt eX = CEIL( bmaxx );
-	TqInt eY = CEIL( bmaxy );
-	if ( eX > xmax ) eX = xmax;
-	if ( eY > ymax ) eY = ymax;
-
-	TqInt sX = FLOOR( bminx );
-	TqInt sY = FLOOR( bminy );
-	if ( sY < ymin ) sY = ymin;
-	if ( sX < xmin ) sX = xmin;
-
-	CqImagePixel* pie, *pie2;
-
-	TqInt iXSamples = PixelXSamples();
-	TqInt iYSamples = PixelYSamples();
-
-	TqInt im = ( bminx < sX ) ? 0 : FLOOR( ( bminx - sX ) * iXSamples );
-	TqInt in = ( bminy < sY ) ? 0 : FLOOR( ( bminy - sY ) * iYSamples );
-	TqInt em = ( bmaxx > eX ) ? iXSamples : CEIL( ( bmaxx - ( eX - 1 ) ) * iXSamples );
-	TqInt en = ( bmaxy > eY ) ? iYSamples : CEIL( ( bmaxy - ( eY - 1 ) ) * iYSamples );
-
-	TqInt nextx = Bucket.RealWidth();
-	Bucket.ImageElement( sX, sY, pie );
-
-	for( int iY = sY; iY < eY; ++iY)
-	{
-		pie2 = pie;
-		pie += nextx;
-
-		for(int iX = sX; iX < eX; ++iX, ++pie2)
-		{
-			// only bother sampling if the mpg is not occluded in this pixel.
-			if(mustDraw || bminz <= pie2->MaxDepth())
-			{
-				if(!cachedHitData)
-				{
-					pMPG->CacheHitTestValues(&hitTestCache);
-					cachedHitData = TqTrue;
-				}
-
-				// Now sample the micropolygon at several subsample positions
-				// within the pixel. The subsample indices range from (start_m, n)
-				// to (end_m-1, end_n-1).
-				register int m, n;
-				n = ( iY == sY ) ? in : 0;
-				int end_n = ( iY == ( eY - 1 ) ) ? en : iYSamples;
-				int start_m = ( iX == sX ) ? im : 0;
-				int end_m = ( iX == ( eX - 1 ) ) ? em : iXSamples;
-				int index_start = n*iXSamples + start_m;
-
-				for ( ; n < end_n; n++ )
-				{
-					int index = index_start;
-					for ( m = start_m; m < end_m; m++, index++ )
-					{
-						const SqSampleData& sampleData = pie2->SampleData( index );
-						const CqVector2D& vecP = sampleData.m_Position;
-						const TqFloat time = 0.0;
-
-						CqStats::IncI( CqStats::SPL_count );
-
-						if(!Bound.Contains2D( vecP ))
-							continue;
-
-						// Check to see if the sample is within the sample's level of detail
-						if ( UsingLevelOfDetail)
-						{
-							TqFloat LevelOfDetail = sampleData.m_DetailLevel;
-							if ( LodBounds[ 0 ] > LevelOfDetail || LevelOfDetail >= LodBounds[ 1 ] )
-							{
-								continue;
-							}
-						}
-
-						CqStats::IncI( CqStats::SPL_bound_hits );
-
-						// Now check if the subsample hits the micropoly
-						TqBool SampleHit;
-						TqFloat D;
-
-						SampleHit = pMPG->Sample( sampleData, D, time );
-
-						if ( SampleHit )
-						{
-							sample_hits++;
-							StoreSample( pMPG, pie2, index, D );
-						}
-					}
-					index_start += iXSamples;
-				}
-			}
-	/*        // Now compute the % of samples that hit...
-			TqInt scount = iXSamples * iYSamples;
-			TqFloat max_hits = scount * shd_rate;
-			TqInt hit_rate = ( sample_hits / max_hits ) / 0.125;
-			STATS_INC( MPG_sample_coverage0_125 + CLAMP( hit_rate - 1 , 0, 7 ) );
-	*/  }
-	}
+	CqOcclusionBox::KDTree()->SampleMPG(pMPG, Bound, TqFalse, 0, 0, TqFalse, 0, m_CurrentMpgSampleInfo, m_CurrentGridInfo.m_LodBounds[0] >= 0.0f, m_CurrentGridInfo);
 }
 
 
 
-void CqImageBuffer::StoreSample( CqMicroPolygon* pMPG, CqImagePixel* pie2, TqInt index, TqFloat D )
-{
-    TqBool Occludes = m_CurrentMpgSampleInfo.m_Occludes;
-	TqBool opaque =  m_CurrentMpgSampleInfo.m_IsOpaque;
-
-	SqImageSample& currentOpaqueSample = pie2->OpaqueValues(index);
-	static SqImageSample localImageVal( QGetRenderContext() ->GetOutputDataTotalSize() );
-
-	SqImageSample& ImageVal = opaque ? currentOpaqueSample : localImageVal;
-
-	std::vector<SqImageSample>& aValues = pie2->Values( index );
-	std::vector<SqImageSample>::iterator sample = aValues.begin();
-	std::vector<SqImageSample>::iterator end = aValues.end();
-
-	// return if the sample is occluded and can be culled.
-	if(opaque)
-	{
-		if((currentOpaqueSample.m_flags & SqImageSample::Flag_Valid) &&
-			currentOpaqueSample.Depth() <= D)
-		{
-			return;
-		}
-	}
-	else
-	{
-		// Sort the color/opacity into the visible point list
-		// return if the sample is occluded and can be culled.
-		while( sample != end )
-		{
-			if((*sample).Depth() >= D)
-				break;
-
-			if(((*sample).m_flags & SqImageSample::Flag_Occludes) &&
-				!(*sample).m_pCSGNode && m_CurrentGridInfo.m_IsCullable)
-				return;
-
-			++sample;
-		}
-	}
-
-    ImageVal.SetDepth( D );
-
-	CqStats::IncI( CqStats::SPL_hits );
-	pMPG->MarkHit();
-
-    std::valarray<TqFloat>& val = ImageVal.m_Data;
-    val[ 0 ] = m_CurrentMpgSampleInfo.m_Colour.fRed();
-    val[ 1 ] = m_CurrentMpgSampleInfo.m_Colour.fGreen();
-    val[ 2 ] = m_CurrentMpgSampleInfo.m_Colour.fBlue();
-    val[ 3 ] = m_CurrentMpgSampleInfo.m_Opacity.fRed();
-    val[ 4 ] = m_CurrentMpgSampleInfo.m_Opacity.fGreen();
-    val[ 5 ] = m_CurrentMpgSampleInfo.m_Opacity.fBlue();
-    val[ 6 ] = D;
-
-    // Now store any other data types that have been registered.
-	if(m_CurrentGridInfo.m_UsesDataMap)
-	{
-		StoreExtraData(pMPG, val);
-	}
-
-	if(!opaque)
-	{
-		// If depth is exactly the same as previous sample, chances are we've
-		// hit a MPG grid line.
-		// \note: Cannot do this if there is CSG involved, as all samples must be taken and kept the same.
-		if ( sample != end && (*sample).Depth() == ImageVal.Depth() && !(*sample).m_pCSGNode )
-		{
-			(*sample).m_Data = ( (*sample).m_Data + val ) * 0.5f;
-			return;
-		}
-	}
-
-    // Update max depth values
-    if ( !( DisplayMode() & ModeZ ) && Occludes )
-    {
-        CqOcclusionBox::MarkForUpdate( pie2->OcclusionBoxId() );
-        pie2->MarkForZUpdate();
-    }
-
-    ImageVal.m_pCSGNode = pMPG->pGrid() ->pCSGNode();
-
-    ImageVal.m_flags = 0;
-    if ( Occludes )
-    {
-        ImageVal.m_flags |= SqImageSample::Flag_Occludes;
-    }
-    if( m_CurrentGridInfo.m_IsMatte )
-    {
-        ImageVal.m_flags |= SqImageSample::Flag_Matte;
-    }
-
-	if(!opaque)
-	{
-		aValues.insert( sample, ImageVal );
-
-		// mark this pixel as using the visible point list.
-		pie2->SetUsesSampleList();
-	}
-	else
-	{
-		// increment the sample count if we havn't already.
-		if(!(ImageVal.m_flags & SqImageSample::Flag_Valid))
-			pie2->IncOpaqueSampleCount();
-
-		// mark this sample as having been written into.
-		ImageVal.m_flags |= SqImageSample::Flag_Valid;
-	}
-}
-
-void CqImageBuffer::StoreExtraData( CqMicroPolygon* pMPG, std::valarray<TqFloat>& val)
+void CqImageBuffer::StoreExtraData( CqMicroPolygon* pMPG, SqImageSample& sample)
 {
     std::map<std::string, CqRenderer::SqOutputDataEntry>& DataMap = QGetRenderContext() ->GetMapOfOutputDataEntries();
 	std::map<std::string, CqRenderer::SqOutputDataEntry>::iterator entry;
@@ -1260,7 +929,7 @@ void CqImageBuffer::StoreExtraData( CqMicroPolygon* pMPG, std::valarray<TqFloat>
                 {
                     TqFloat f;
                     pData->GetFloat( f, pMPG->GetIndex() );
-                    val[ entry->second.m_Offset ] = f;
+                    sample.Data()[ entry->second.m_Offset ] = f;
                     break;
                 }
             case type_point:
@@ -1270,18 +939,18 @@ void CqImageBuffer::StoreExtraData( CqMicroPolygon* pMPG, std::valarray<TqFloat>
                 {
                     CqVector3D v;
                     pData->GetPoint( v, pMPG->GetIndex() );
-                    val[ entry->second.m_Offset ] = v.x();
-                    val[ entry->second.m_Offset + 1 ] = v.y();
-                    val[ entry->second.m_Offset + 2 ] = v.z();
+                    sample.Data()[ entry->second.m_Offset ] = v.x();
+                    sample.Data()[ entry->second.m_Offset + 1 ] = v.y();
+                    sample.Data()[ entry->second.m_Offset + 2 ] = v.z();
                     break;
                 }
             case type_color:
                 {
                     CqColor c;
                     pData->GetColor( c, pMPG->GetIndex() );
-                    val[ entry->second.m_Offset ] = c.fRed();
-                    val[ entry->second.m_Offset + 1 ] = c.fGreen();
-                    val[ entry->second.m_Offset + 2 ] = c.fBlue();
+                    sample.Data()[ entry->second.m_Offset ] = c.fRed();
+                    sample.Data()[ entry->second.m_Offset + 1 ] = c.fGreen();
+                    sample.Data()[ entry->second.m_Offset + 2 ] = c.fBlue();
                     break;
                 }
             case type_matrix:
@@ -1289,22 +958,22 @@ void CqImageBuffer::StoreExtraData( CqMicroPolygon* pMPG, std::valarray<TqFloat>
                     CqMatrix m;
                     pData->GetMatrix( m, pMPG->GetIndex() );
                     TqFloat* pElements = m.pElements();
-                    val[ entry->second.m_Offset ] = pElements[ 0 ];
-                    val[ entry->second.m_Offset + 1 ] = pElements[ 1 ];
-                    val[ entry->second.m_Offset + 2 ] = pElements[ 2 ];
-                    val[ entry->second.m_Offset + 3 ] = pElements[ 3 ];
-                    val[ entry->second.m_Offset + 4 ] = pElements[ 4 ];
-                    val[ entry->second.m_Offset + 5 ] = pElements[ 5 ];
-                    val[ entry->second.m_Offset + 6 ] = pElements[ 6 ];
-                    val[ entry->second.m_Offset + 7 ] = pElements[ 7 ];
-                    val[ entry->second.m_Offset + 8 ] = pElements[ 8 ];
-                    val[ entry->second.m_Offset + 9 ] = pElements[ 9 ];
-                    val[ entry->second.m_Offset + 10 ] = pElements[ 10 ];
-                    val[ entry->second.m_Offset + 11 ] = pElements[ 11 ];
-                    val[ entry->second.m_Offset + 12 ] = pElements[ 12 ];
-                    val[ entry->second.m_Offset + 13 ] = pElements[ 13 ];
-                    val[ entry->second.m_Offset + 14 ] = pElements[ 14 ];
-                    val[ entry->second.m_Offset + 15 ] = pElements[ 15 ];
+                    sample.Data()[ entry->second.m_Offset ] = pElements[ 0 ];
+                    sample.Data()[ entry->second.m_Offset + 1 ] = pElements[ 1 ];
+                    sample.Data()[ entry->second.m_Offset + 2 ] = pElements[ 2 ];
+                    sample.Data()[ entry->second.m_Offset + 3 ] = pElements[ 3 ];
+                    sample.Data()[ entry->second.m_Offset + 4 ] = pElements[ 4 ];
+                    sample.Data()[ entry->second.m_Offset + 5 ] = pElements[ 5 ];
+                    sample.Data()[ entry->second.m_Offset + 6 ] = pElements[ 6 ];
+                    sample.Data()[ entry->second.m_Offset + 7 ] = pElements[ 7 ];
+                    sample.Data()[ entry->second.m_Offset + 8 ] = pElements[ 8 ];
+                    sample.Data()[ entry->second.m_Offset + 9 ] = pElements[ 9 ];
+                    sample.Data()[ entry->second.m_Offset + 10 ] = pElements[ 10 ];
+                    sample.Data()[ entry->second.m_Offset + 11 ] = pElements[ 11 ];
+                    sample.Data()[ entry->second.m_Offset + 12 ] = pElements[ 12 ];
+                    sample.Data()[ entry->second.m_Offset + 13 ] = pElements[ 13 ];
+                    sample.Data()[ entry->second.m_Offset + 14 ] = pElements[ 14 ];
+                    sample.Data()[ entry->second.m_Offset + 15 ] = pElements[ 15 ];
                     break;
                 }
             default:
@@ -1347,20 +1016,15 @@ void CqImageBuffer::StoreExtraData( CqMicroPolygon* pMPG, std::valarray<TqFloat>
  * \param ymax Integer maximum extend of the image part being rendered, takes into account buckets and clipping.
  */
 
-void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax )
+void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax, TqBool fImager, enum EqFilterDepth depthfilter, CqColor zThreshold)
 {
 	TqBool bIsEmpty = IsCurrentBucketEmpty();
 
     // Render any waiting micro polygon grids.
-    QGetRenderContext() ->Stats().RenderMPGsTimer().Start();
-    RenderMPGs( xmin, xmax, ymin, ymax );
-    QGetRenderContext() ->Stats().RenderMPGsTimer().Stop();
-
-    QGetRenderContext() ->Stats().OcclusionCullTimer().Start();
-    // Update our occlusion hierarchy after drawing.
-    if ( !bIsEmpty )
-        CqOcclusionBox::Update();
-    QGetRenderContext() ->Stats().OcclusionCullTimer().Stop();
+	{
+	    TIME_SCOPE("Render MPGs")
+		RenderMPGs( xmin, xmax, ymin, ymax );
+	}
 
 	CqBucket& Bucket = CurrentBucket();
 
@@ -1370,36 +1034,40 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax )
     {
         if ( m_fQuit ) return ;
 
+
+        //Cull surface if it's hidden
+        if ( !( DisplayMode() & ModeZ ) && !pSurface->pCSGNode() )
+        {
+            TIME_SCOPE("Occlusion culling")
+            TqBool fCull = TqFalse;
+			if ( !bIsEmpty && pSurface->fCachedBound() )
+				fCull = OcclusionCullSurface( pSurface );
+            if ( fCull )
+            {
+				Bucket.popSurface();
+                pSurface = Bucket.pTopSurface();
+                continue;
+            }
+        }
+
         // If the epsilon check has deemed this surface to be undiceable, don't bother asking.
         TqBool fDiceable = TqFalse;
         // Dice & shade the surface if it's small enough...
-        QGetRenderContext() ->Stats().DiceableTimer().Start();
-        fDiceable = pSurface->Diceable();
-        QGetRenderContext() ->Stats().DiceableTimer().Stop();
+        {
+			TIME_SCOPE("Dicable check")
+			fDiceable = pSurface->Diceable();
+		}
 
         if ( fDiceable )
         {
-            //Cull surface if it's hidden
-            if ( !( DisplayMode() & ModeZ ) && !pSurface->pCSGNode() )
-            {
-                QGetRenderContext() ->Stats().OcclusionCullTimer().Start();
-                TqBool fCull = TqFalse;
-                if ( !bIsEmpty && pSurface->fCachedBound() )
-                    fCull = OcclusionCullSurface( pSurface );
-                QGetRenderContext() ->Stats().OcclusionCullTimer().Stop();
-                if ( fCull )
-                {
-					Bucket.popSurface();
-                    pSurface = Bucket.pTopSurface();
-                    continue;
-                }
-            }
 
 		    Bucket.popSurface();	
             CqMicroPolyGridBase* pGrid;
-            QGetRenderContext() ->Stats().DicingTimer().Start();
-            pGrid = pSurface->Dice();
-            QGetRenderContext() ->Stats().DicingTimer().Stop();
+            {
+				TIME_SCOPE("Dicing")
+			    pGrid = pSurface->Dice();
+			}
+
             if ( NULL != pGrid )
             {
                 ADDREF( pGrid );
@@ -1412,9 +1080,10 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax )
                     // Only project micropolygon not culled
                     Bucket.AddGrid( pGrid );
                     // Render any waiting micro polygon grids.
-                    QGetRenderContext() ->Stats().RenderMPGsTimer().Start();
-                    RenderMPGs( xmin, xmax, ymin, ymax );
-                    QGetRenderContext() ->Stats().RenderMPGsTimer().Stop();
+                    {
+						TIME_SCOPE("Render MPGs")
+						RenderMPGs( xmin, xmax, ymin, ymax );
+					}
                 }
                 RELEASEREF( pGrid );
             }
@@ -1428,15 +1097,28 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax )
             STATS_DEC( GPR_created_total );
 
             // Split it
-            QGetRenderContext() ->Stats().SplitsTimer().Start();
-            std::vector<boost::shared_ptr<CqBasicSurface> > aSplits;
-            TqInt cSplits = pSurface->Split( aSplits );
-            TqInt i;
-            for ( i = 0; i < cSplits; i++ )
-                PostSurface( aSplits[ i ] );
+            {
+				TIME_SCOPE("Splits")
+				std::vector<boost::shared_ptr<CqBasicSurface> > aSplits;
+				TqInt cSplits = pSurface->Split( aSplits );
+				TqInt i;
+				for ( i = 0; i < cSplits; i++ )
+					PostSurface( aSplits[ i ] );
 
-            QGetRenderContext() ->Stats().SplitsTimer().Stop();
-        } else if ( pSurface == Bucket.pTopSurface() ) 
+				/// \debug:
+/*				if(pSurface->IsUndiceable())
+				{
+				    CqBound Bound( pSurface->Bound() );
+					std::cout << pSurface << " - " << Bound.vecMin().z() << " --> " << Bound.vecMax().z() << std::endl;
+					for ( i = 0; i < cSplits; i++ )
+					{
+					    CqBound SBound( aSplits[i]->Bound() );
+						std::cout << "\t" << aSplits[i] << " - " << SBound.vecMin().z() << " --> " << SBound.vecMax().z() << std::endl;
+					}
+				}
+*/			}
+        } 
+		else if ( pSurface == Bucket.pTopSurface() ) 
         {
              // Make sure we will break the while() 
              //   e.g.  !fDiceable  &&  pSurface->fDiscard()
@@ -1445,15 +1127,10 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax )
 
         pSurface = Bucket.pTopSurface();
         // Render any waiting micro polygon grids.
-        QGetRenderContext() ->Stats().RenderMPGsTimer().Start();
-        RenderMPGs( xmin, xmax, ymin, ymax );
-        QGetRenderContext() ->Stats().RenderMPGsTimer().Stop();
-
-        QGetRenderContext() ->Stats().OcclusionCullTimer().Start();
-        // Update our occlusion hierarchy after each grid that gets drawn.
-        if ( !bIsEmpty )
-            CqOcclusionBox::Update();
-        QGetRenderContext() ->Stats().OcclusionCullTimer().Stop();
+        {
+			TIME_SCOPE("Render MPGs")
+			RenderMPGs( xmin, xmax, ymin, ymax );
+		}
     }
 
     // Now combine the colors at each pixel sample for any micropolygons rendered to that pixel.
@@ -1461,19 +1138,11 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax )
 
     if(!bIsEmpty)
     {
-        QGetRenderContext() ->Stats().MakeCombine().Start();
-        CqBucket::CombineElements();
-        QGetRenderContext() ->Stats().MakeCombine().Stop();
+        TIME_SCOPE("Combine")
+        CqBucket::CombineElements(depthfilter, zThreshold);
     }
 
-    QGetRenderContext() ->Stats().MakeFilterBucket().Start();
-
-    TqBool fImager = TqFalse;
-    const CqString* systemOptions;
-    if( ( systemOptions = QGetRenderContext() ->optCurrent().GetStringOption( "System", "Imager" ) ) != 0 )
-    	if( systemOptions[ 0 ].compare("null") != 0 )
-		fImager = TqTrue;
-
+	TIMER_START("Filter")
     if (fImager)
         bIsEmpty = TqFalse;
 
@@ -1484,12 +1153,13 @@ void CqImageBuffer::RenderSurfaces( long xmin, long xmax, long ymin, long ymax )
 	Bucket.QuantizeBucket();
     }
 
-    QGetRenderContext() ->Stats().MakeFilterBucket().Stop();
+	TIMER_STOP("Filter")
 
     BucketComplete();
-    QGetRenderContext() ->Stats().MakeDisplayBucket().Start();
-    QGetRenderContext() ->pDDmanager() ->DisplayBucket( &CurrentBucket() );
-    QGetRenderContext() ->Stats().MakeDisplayBucket().Stop();
+    {
+		TIME_SCOPE("Display bucket")
+		QGetRenderContext() ->pDDmanager() ->DisplayBucket( &CurrentBucket() );
+	}
 }
 
 //----------------------------------------------------------------------
@@ -1548,40 +1218,61 @@ void CqImageBuffer::RenderImage()
         if ( bucketmodulo <= 0 ) bucketmodulo = m_cXBuckets;
     }
 
+    TqBool fImager = TqFalse;
+    const CqString* systemOptions;
+    if( ( systemOptions = QGetRenderContext() ->optCurrent().GetStringOption( "System", "Imager" ) ) != 0 )
+    if( systemOptions[ 0 ].compare("null") != 0 )
+		fImager = TqTrue;
 
+    const CqString* pstrDepthFilter = QGetRenderContext() ->optCurrent().GetStringOption( "Hider", "depthfilter" );
+    const CqColor* pzThreshold = QGetRenderContext() ->optCurrent().GetColorOption( "limits", "zthreshold" );
+    CqColor zThreshold(1.0f, 1.0f, 1.0f);	// Default threshold of 1,1,1 means that any objects that are partially transparent won't appear in shadow maps.
+    if(NULL != pzThreshold)
+	zThreshold = pzThreshold[0];
+    enum EqFilterDepth depthfilter = Filter_Min;
+    if ( NULL != pstrDepthFilter )
+    {
+	if( !pstrDepthFilter[ 0 ].compare( "min" ) )
+		depthfilter = Filter_Min;
+	else if ( !pstrDepthFilter[ 0 ].compare( "midpoint" ) )
+		depthfilter = Filter_MidPoint;
+	else if ( !pstrDepthFilter[ 0 ].compare( "max" ) )
+		depthfilter = Filter_Max;
+	else if ( !pstrDepthFilter[ 0 ].compare( "average" ) )
+		depthfilter = Filter_Average;
+	else
+		std::cerr << warning << "Invalid depthfilter \"" << pstrDepthFilter[ 0 ].c_str() << "\", depthfilter set to \"min\"" << std::endl;
+    }
 
     // Render the surface at the front of the list.
     m_fDone = TqFalse;
 
     CqVector2D bHalf = CqVector2D( FLOOR(m_FilterXWidth / 2.0f), FLOOR(m_FilterYWidth / 2.0f) );
 
-    // Setup the hierarchy of boxes for occlusion culling
-    CqOcclusionBox::CreateHierarchy( m_XBucketSize, m_YBucketSize, m_FilterXWidth, m_FilterYWidth );
-
     RtProgressFunc pProgressHandler = NULL;
     pProgressHandler = QGetRenderContext()->pProgressHandler();
 
     do
     {
         TqBool bIsEmpty = IsCurrentBucketEmpty();
-        QGetRenderContext() ->Stats().Others().Start();
         // Prepare the bucket.
         CqVector2D bPos = BucketPosition();
         CqVector2D bSize = BucketSize();
-        // TODO: fix non jittered bucket initialisation.
-        // Warning Jitter must be True is all cases; the InitialiseBucket when it is not in jittering mode
-        // doesn't initialise correctly so later we have problem in the FilterBucket()
+
         TqBool fImager = TqFalse;
         const CqString* systemOptions;
         if( ( systemOptions = QGetRenderContext() ->optCurrent().GetStringOption( "System", "Imager" ) ) != 0 )
     	if( systemOptions[ 0 ].compare("null") != 0 )
-		fImager = TqTrue;
+			fImager = TqTrue;
 
         if (fImager)
             bIsEmpty = TqFalse;
 
-        CqBucket::InitialiseBucket( static_cast<TqInt>( bPos.x() ), static_cast<TqInt>( bPos.y() ), static_cast<TqInt>( bSize.x() ), static_cast<TqInt>( bSize.y() ), true, bIsEmpty );
-        CqBucket::InitialiseFilterValues();
+        {
+			TIME_SCOPE("Prepare bucket")
+			CqBucket::PrepareBucket( static_cast<TqInt>( bPos.x() ), static_cast<TqInt>( bPos.y() ), static_cast<TqInt>( bSize.x() ), static_cast<TqInt>( bSize.y() ), true, bIsEmpty );
+			CqBucket::InitialiseFilterValues();
+		}
 
 		////////// Dump the pixel sample positions into a dump file //////////
 		#ifdef DEBUG_MPDUMP
@@ -1609,13 +1300,11 @@ void CqImageBuffer::RenderImage()
         if ( ymax > CropWindowYMax() + m_FilterYWidth / 2 )
 			ymax = static_cast<long>(CropWindowYMax() + m_FilterYWidth / 2.0f);
 
-        QGetRenderContext() ->Stats().Others().Stop();
 
         if ( !bIsEmpty )
         {
-            QGetRenderContext() ->Stats().OcclusionCullTimer().Start();
+            TIME_SCOPE("Occlusion culling")
             CqOcclusionBox::SetupHierarchy( &CurrentBucket(), xmin, ymin, xmax, ymax );
-            QGetRenderContext() ->Stats().OcclusionCullTimer().Stop();
         }
 
 
@@ -1632,7 +1321,7 @@ void CqImageBuffer::RenderImage()
         }
 
 
-        RenderSurfaces( xmin, xmax, ymin, ymax );
+        RenderSurfaces( xmin, xmax, ymin, ymax, fImager, depthfilter, zThreshold );
         if ( m_fQuit )
         {
             m_fDone = TqTrue;
