@@ -42,10 +42,36 @@
 #include	"vector2d.h"
 #include    "imagepixel.h"
 #include    "bucket.h"
+#include	"kdtree.h"
+#include	"occlusion.h"
+#include	"clippingvolume.h"
 
 START_NAMESPACE( Aqsis )
 
 
+// This struct is used to hold info about a mpg that is used when rendering the mpg.
+// It caches the info for use by multiple samples.
+struct SqMpgSampleInfo
+{
+    CqColor		m_Colour;
+    CqColor		m_Opacity;
+    TqBool		m_Occludes;		// whether the opacity is full.
+	TqBool		m_IsOpaque;		// whether the mpg can use the faster StoreOpaqueSample routine that assumes a few things.
+};
+
+// This struct holds info about a grid that can be cached and used for all its mpgs.
+struct SqGridInfo
+{
+	TqFloat			m_ShadingRate;
+	TqFloat			m_ShutterOpenTime;
+	TqFloat			m_ShutterCloseTime;
+	const TqFloat*	        m_LodBounds;
+	TqBool			m_IsMatte;
+	TqBool			m_IsCullable;
+	TqBool			m_UsesDataMap;
+};
+
+
 
 //-----------------------------------------------------------------------
 /**
@@ -91,7 +117,8 @@ public:
             m_CropWindowYMax( 0 ),
             m_DisplayMode( ModeRGB ),
             m_CurrentBucketCol( 0 ),
-            m_CurrentBucketRow( 0 )
+            m_CurrentBucketRow( 0 ),
+            m_MaxEyeSplits(10)
     {}
     virtual	~CqImageBuffer();
 
@@ -271,12 +298,12 @@ public:
     TqBool	PushMPGDown( CqMicroPolygon*, TqInt Col, TqInt Row );
     void	RenderMPGs( long xmin, long xmax, long ymin, long ymax );
     void	RenderMicroPoly( CqMicroPolygon* pMPG, long xmin, long xmax, long ymin, long ymax );
+    void	RenderSurfaces( long xmin, long xmax, long ymin, long ymax, TqBool fImager, enum EqFilterDepth filterdepth, CqColor zThreshold );
+    void	RenderImage();
+	void	StoreExtraData( CqMicroPolygon* pMPG, SqImageSample& sample);
+
 	void	RenderMPG_MBOrDof( CqMicroPolygon* pMPG, long xmin, long xmax, long ymin, long ymax, TqBool IsMoving, TqBool UsingDof );
 	void	RenderMPG_Static( CqMicroPolygon* pMPG, long xmin, long xmax, long ymin, long ymax );
-    void	RenderSurfaces( long xmin, long xmax, long ymin, long ymax );
-    void	RenderImage();
-    void	StoreSample( CqMicroPolygon* pMPG, CqImagePixel* pie2, TqInt index, TqFloat D );
-	void	StoreExtraData( CqMicroPolygon* pMPG, std::valarray<TqFloat>& val);
 
 
 
@@ -300,6 +327,7 @@ public:
     {}
     virtual	TqBool	IsCurrentBucketEmpty();
 
+
 private:
     TqBool	m_fQuit;			///< Set by system if a quit has been requested.
     TqBool	m_fDone;			///< Set when the render of this image has completed.
@@ -325,34 +353,17 @@ private:
     std::vector<std::vector<CqBucket> >	m_Buckets; ///< Array of bucket storage classes (row/col)
     TqInt	m_CurrentBucketCol;	///< Column index of the bucket currently being processed.
     TqInt	m_CurrentBucketRow;	///< Row index of the bucket currently being processed.
+    TqInt	m_MaxEyeSplits;	        ///< Max Eye Splits by default 10
 
-    // This struct is used to hold info about a mpg that is used when rendering the mpg.
-    // It caches the info for use by multiple samples.
-    struct SqMpgSampleInfo
-    {
-        CqColor		m_Colour;
-        CqColor		m_Opacity;
-        TqBool		m_Occludes;		// whether the opacity is full.
-		TqBool		m_IsOpaque;		// whether the mpg can use the faster StoreOpaqueSample routine that assumes a few things.
-    };
     SqMpgSampleInfo m_CurrentMpgSampleInfo;
 
-	// This struct holds info about a grid that can be cached and used for all its mpgs.
-	struct SqGridInfo
-	{
-		TqFloat			m_ShadingRate;
-		TqFloat			m_ShutterOpenTime;
-		TqFloat			m_ShutterCloseTime;
-		TqBool			m_IsMatte;
-		TqBool			m_IsCullable;
-		TqBool			m_UsesDataMap;
-		const TqFloat*	m_LodBounds;
-	};
 	SqGridInfo m_CurrentGridInfo;
 
 	void CacheGridInfo( CqMicroPolyGridBase* pGrid );
 };
 
+
+
 //-----------------------------------------------------------------------
 
 END_NAMESPACE( Aqsis )
