@@ -29,7 +29,7 @@
 
 START_NAMESPACE( Aqsis )
 
-DEFINE_STATIC_MEMORYPOOL( CqLath, 512 );
+CqObjectPool<CqLath>	CqLath::m_thePool;
 
 //------------------------------------------------------------------------------
 /**
