@@ -35,11 +35,13 @@
 
 #include	"mpdump.h"
 
+#include	"MultiTimer.h"
+
 START_NAMESPACE( Aqsis )
 
 
-DEFINE_STATIC_MEMORYPOOL( CqMicroPolygon, 512 );
-DEFINE_STATIC_MEMORYPOOL( CqMovingMicroPolygonKey, 512 );
+CqObjectPool<CqMicroPolygon> CqMicroPolygon::m_thePool;
+CqObjectPool<CqMovingMicroPolygonKey>	CqMovingMicroPolygonKey::m_thePool;
 
 //---------------------------------------------------------------------
 /** Default constructor
@@ -130,7 +132,7 @@ void CqMicroPolyGrid::Initialise( TqInt cu, TqInt cv, const boost::shared_ptr<Cq
 
     TqInt size = ( cu + 1 ) * ( cv + 1 );
 
-    STATS_INC( GRD_size_4 + CLAMP( CqStats::log2( size ) - 2, 0, 7 ) );
+    STATS_INC( GRD_size_4 + CLAMP( CqStats::stats_log2( size ) - 2, 0, 7 ) );
 }
 
 //---------------------------------------------------------------------
@@ -302,39 +304,14 @@ void CqMicroPolyGrid::Shade()
     {
 	    if ( USES( lUses, EnvVars_du ) )
 		{
-            TqFloat v1, v2;
-            TqInt GridX = i % ( uRes + 1 );
 
-            if ( GridX < uRes )
-            {
-                pVar(EnvVars_u) ->GetValue( v1, i + 1 );
-                pVar(EnvVars_u) ->GetValue( v2, i );
-                pVar(EnvVars_du) ->SetFloat( 1.0f/*v1 - v2, i*/ );
-            }
-            else
-            {
-                pVar(EnvVars_u) ->GetValue( v1, i );
-                pVar(EnvVars_u) ->GetValue( v2, i - 1 );
-                pVar(EnvVars_du) ->SetFloat( 1.0f/*v1 - v2, i*/ );
-            }
+            pVar(EnvVars_du) ->SetFloat( 1.0f, i );
+
         }
 		if ( USES( lUses, EnvVars_dv ) )
 		{
-            TqFloat v1, v2;
-            TqInt GridY = ( i / ( uRes + 1 ) );
 
-            if ( GridY < vRes )
-            {
-                pVar(EnvVars_v) ->GetValue( v1, i + uRes + 1 );
-                pVar(EnvVars_v) ->GetValue( v2, i );
-                pVar(EnvVars_dv) ->SetFloat( 1.0f/*v1 - v2, i*/ );
-            }
-            else
-            {
-                pVar(EnvVars_v) ->GetValue( v1, i );
-                pVar(EnvVars_v) ->GetValue( v2, i - ( uRes + 1 ) );
-                pVar(EnvVars_dv) ->SetFloat( 1.0f/*v1 - v2, i*/ );
-            }
+            pVar(EnvVars_dv) ->SetFloat( 1.0f, i );
         }
 		switch ( proj )
 		{
@@ -373,8 +350,10 @@ void CqMicroPolyGrid::Shade()
     cCulled = 0;
     if ( USES( lUses, EnvVars_Os ) && QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "DisplayMode" ) [ 0 ] & ModeZ )
     {
-        theStats.OcclusionCullTimer().Start();
-        for ( i = gsmin1; i >= 0; i-- )
+        //theStats.OcclusionCullTimer().Start();
+        TIME_SCOPE("Occlusion Culling")
+		{
+		for ( i = gsmin1; i >= 0; i-- )
         {
             if ( pOs[ i ] != gColWhite )
             {
@@ -384,7 +363,8 @@ void CqMicroPolyGrid::Shade()
             else
                 break;
         }
-        theStats.OcclusionCullTimer().Stop();
+        //theStats.OcclusionCullTimer().Stop();
+		}
 
         if ( cCulled == gs )
         {
@@ -401,8 +381,9 @@ void CqMicroPolyGrid::Shade()
     cCulled = 0;
     if ( USES( lUses, EnvVars_Os ) && QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "DisplayMode" ) [ 0 ] & ModeRGB )
     {
-        theStats.OcclusionCullTimer().Start();
-        for ( i = gsmin1; i >= 0; i-- )
+        //theStats.OcclusionCullTimer().Start();
+        TIME_SCOPE("Occlusion culling")
+		for ( i = gsmin1; i >= 0; i-- )
         {
             if ( pOs[ i ] == gColBlack )
             {
@@ -412,7 +393,7 @@ void CqMicroPolyGrid::Shade()
             else
                 break;
         }
-        theStats.OcclusionCullTimer().Stop();
+        //theStats.OcclusionCullTimer().Stop();
 
         if ( cCulled == gs )
         {
@@ -425,17 +406,19 @@ void CqMicroPolyGrid::Shade()
 
     if ( pshadDisplacement )
     {
-        theStats.DisplacementTimer().Start();
-        pshadDisplacement->Evaluate( m_pShaderExecEnv );
-        theStats.DisplacementTimer().Stop();
+        //theStats.DisplacementTimer().Start();
+        TIME_SCOPE("Discplacement")
+		pshadDisplacement->Evaluate( m_pShaderExecEnv );
+        //theStats.DisplacementTimer().Stop();
     }
 
     // Now try and cull any hidden MPs if Sides==1
     cCulled = 0;
     if ( ( pAttributes() ->GetIntegerAttribute( "System", "Sides" ) [ 0 ] == 1 ) && !m_pCSGNode )
     {
-        theStats.OcclusionCullTimer().Start();
-        for ( i = gsmin1; i >= 0; i-- )
+        //theStats.OcclusionCullTimer().Start();
+        TIME_SCOPE("Occlusion culling")
+		for ( i = gsmin1; i >= 0; i-- )
         {
             // Calulate the direction the MPG is facing.
             if ( ( pN[ i ] * pP[ i ] ) >= 0 )
@@ -444,7 +427,7 @@ void CqMicroPolyGrid::Shade()
                 m_CulledPolys.SetValue( i, TqTrue );
             }
         }
-        theStats.OcclusionCullTimer().Stop();
+        //theStats.OcclusionCullTimer().Stop();
 
         // If the whole grid is culled don't bother going any further.
         if ( cCulled == gs )
@@ -457,22 +440,24 @@ void CqMicroPolyGrid::Shade()
     }
 
     // Now shade the grid.
-    theStats.SurfaceTimer().Start();
+    //theStats.SurfaceTimer().Start();
     if ( pshadSurface )
     {
+		TIME_SCOPE("Surface shading")
 		//boost::shared_ptr<CqSurface> surf(pSurface());
 		m_pShaderExecEnv->SetCurrentSurface(pSurface());
         pshadSurface->Evaluate( m_pShaderExecEnv );
     }
-    theStats.SurfaceTimer().Stop();
+    //theStats.SurfaceTimer().Stop();
 
     // Now try and cull any true transparent MPs (assigned by the shader code
 
     cCulled = 0;
     if ( USES( lUses, EnvVars_Os ) && QGetRenderContext() ->optCurrent().GetIntegerOption( "System", "DisplayMode" ) [ 0 ] & ModeRGB )
     {
-        theStats.OcclusionCullTimer().Start();
-        for ( i = gsmin1; i >= 0; i-- )
+        //theStats.OcclusionCullTimer().Start();
+        TIME_SCOPE("Occlusion culling")
+		for ( i = gsmin1; i >= 0; i-- )
         {
             if ( pOs[ i ] == gColBlack )
             {
@@ -482,7 +467,7 @@ void CqMicroPolyGrid::Shade()
             else
                 break;
         }
-        theStats.OcclusionCullTimer().Stop();
+        //theStats.OcclusionCullTimer().Stop();
 
         if ( cCulled == gs )
         {
@@ -495,16 +480,17 @@ void CqMicroPolyGrid::Shade()
     // Now perform atmosphere shading
     if ( pshadAtmosphere )
     {
-        theStats.AtmosphereTimer().Start();
+        TIME_SCOPE("Atmosphere shading")
+		//theStats.AtmosphereTimer().Start();
         pshadAtmosphere->Evaluate( m_pShaderExecEnv );
-        theStats.AtmosphereTimer().Stop();
+        //theStats.AtmosphereTimer().Stop();
     }
 
     DeleteVariables( TqFalse );
 
     TqInt	size = GridSize();
 
-    STATS_INC( GRD_shd_size_4 + CLAMP( CqStats::log2( size ) - 2, 0, 7 ) );
+    STATS_INC( GRD_shd_size_4 + CLAMP( CqStats::stats_log2( size ) - 2, 0, 7 ) );
 }
 
 //---------------------------------------------------------------------
@@ -610,8 +596,8 @@ void CqMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax, long y
     TqInt cu = uGridRes();
     TqInt cv = vGridRes();
 
-	QGetRenderContext() ->Stats().MakeProject().Start();
-    CqMatrix matCameraToRaster = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", CqMatrix(), CqMatrix(), QGetRenderContext()->Time() );
+    TIMER_START("Project points")
+	CqMatrix matCameraToRaster = QGetRenderContext() ->matSpaceToSpace( "camera", "raster", CqMatrix(), CqMatrix(), QGetRenderContext()->Time() );
     CqMatrix matCameraToObject0 = QGetRenderContext() ->matSpaceToSpace( "camera", "object", CqMatrix(), pSurface() ->pTransform() ->matObjectToWorld( pSurface() ->pTransform() ->Time( 0 ) ), QGetRenderContext()->Time() );
 
     // Transform the whole grid to hybrid camera/raster space
@@ -694,7 +680,7 @@ void CqMicroPolyGrid::Split( CqImageBuffer* pImage, long xmin, long xmax, long y
 	}
 	m_TriangleSplitLine.AddTimeSlot(pSurface()->pTransform()->Time( 0 ), sl ); 
 
-    QGetRenderContext() ->Stats().MakeProject().Stop();
+	TIMER_STOP("Project points")
 
     // Get the required trim curve sense, if specified, defaults to "inside".
     const CqString* pattrTrimSense = pAttributes() ->GetStringAttribute( "trimcurve", "sense" );
