@@ -31,7 +31,7 @@
 
 #include	"aqsis.h"
 
-#include	"memorypool.h"
+#include	"pool.h"
 #include	"color.h"
 #include	"list.h"
 #include	"bound.h"
@@ -87,6 +87,7 @@ public:
 
     virtual	const IqAttributes* pAttributes() const = 0;
 
+	virtual TqBool	usesCSG() const = 0;
     virtual	boost::shared_ptr<CqCSGTreeNode> pCSGNode() const = 0;
     TqBool vfCulled()
     {
@@ -224,6 +225,11 @@ public:
         assert( m_pShaderExecEnv );
         return ( m_pShaderExecEnv->pAttributes() );
     }
+	virtual TqBool	usesCSG() const
+	{
+	TqBool result = (m_pCSGNode.get() != NULL);
+		return(result);
+	}
     virtual	boost::shared_ptr<CqCSGTreeNode> pCSGNode() const
     {
         return ( m_pCSGNode );
@@ -343,6 +349,10 @@ public:
         return ( static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->pAttributes() );
     }
 
+	virtual TqBool	usesCSG() const
+	{
+		return(static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->usesCSG());
+	}
     virtual boost::shared_ptr<CqCSGTreeNode> pCSGNode() const
     {
         return ( static_cast<CqMicroPolyGrid*>( GetMotionObject( Time( 0 ) ) ) ->pCSGNode() );
@@ -394,11 +404,26 @@ struct CqHitTestCache
  * Abstract base class from which static and motion micropolygons are derived.
  */
 
-class CqMicroPolygon : public CqRefCount, public CqPoolable<CqMicroPolygon, 512>
+class CqMicroPolygon : public CqRefCount
 {
 public:
     CqMicroPolygon();
     virtual	~CqMicroPolygon();
+
+    /** Overridden operator new to allocate micropolys from a pool.
+     */
+    void* operator new( size_t size )
+    {
+        return( m_thePool.alloc() );
+    }
+
+    /** Overridden operator delete to allocate micropolys from a pool.
+     */
+    void operator delete( void* p )
+    {
+        m_thePool.free( reinterpret_cast<CqMicroPolygon*>(p) );
+    }
+
 #ifdef _DEBUG
     CqString className() const { return CqString("CqMicroPolygon"); }
 #endif
@@ -470,20 +495,20 @@ public:
     /** Get the color of this micropoly.
      * \return CqColor reference.
      */
-    const	CqColor	colColor() const
+    const CqColor*	colColor() const
     {
-        CqColor colRes;
-        m_pGrid->pVar(EnvVars_Ci) ->GetColor( colRes, m_Index );
-        return ( colRes );
+        CqColor* col;
+        m_pGrid->pVar(EnvVars_Ci) ->GetColorPtr( col );
+        return ( &col[m_Index] );
     }
     /** Get the opacity of this micropoly.
      * \return CqColor reference.
      */
-    const	CqColor	colOpacity() const
+    const	CqColor* colOpacity() const
     {
-        CqColor colRes;
-        m_pGrid->pVar(EnvVars_Oi) ->GetColor( colRes, m_Index );
-        return ( colRes );
+        CqColor* col;
+        m_pGrid->pVar(EnvVars_Oi) ->GetColorPtr( col );
+        return ( &col[m_Index] );
     }
 
     /** Calculate the bound of the micropoly.
@@ -494,18 +519,15 @@ public:
     /** Get the bound of the micropoly.
      * \return CqBound representing the conservative bound.
      */
-    virtual	CqBound GetTotalBound()
+    virtual	CqBound& GetTotalBound()
+	{
+		return m_Bound;
+	}
+    virtual	const CqBound& GetTotalBound() const
 	{
 		return m_Bound;
 	}
 
-    /** Get the bound of the micropoly.
-     * \return CqBound representing the conservative bound.
-     */
-    virtual const CqBound GetTotalBound() const
-    {
-        return ( GetTotalBound() );
-    }
     virtual	TqInt	cSubBounds()
     {
         return ( 1 );
@@ -626,6 +648,8 @@ protected:
 private:
     CqMicroPolygon( const CqMicroPolygon& From )
 {}
+
+	static	CqObjectPool<CqMicroPolygon> m_thePool;
 }
 ;
 
@@ -636,7 +660,7 @@ private:
  * Base lass for static micropolygons. Stores point information about the geometry of the micropoly.
  */
 
-class CqMovingMicroPolygonKey : public CqPoolable<CqMovingMicroPolygonKey, 512>
+class CqMovingMicroPolygonKey
 {
 public:
     CqMovingMicroPolygonKey()
@@ -648,6 +672,21 @@ public:
     ~CqMovingMicroPolygonKey()
     {}
 
+    /** Overridden operator new to allocate micropolys from a pool.
+     */
+    void* operator new( size_t size )
+    {
+        return( m_thePool.alloc() );
+    }
+
+    /** Overridden operator delete to allocate micropolys from a pool.
+     */
+    void operator delete( void* p )
+    {
+        m_thePool.free( reinterpret_cast<CqMovingMicroPolygonKey*>(p) );
+    }
+
+
 public:
     const CqBound&	GetTotalBound();
     void	Initialise( const CqVector3D& vA, const CqVector3D& vB, const CqVector3D& vC, const CqVector3D& vD );
@@ -667,6 +706,8 @@ public:
 
 	CqBound m_Bound;
 	TqBool	m_BoundReady;
+
+	static	CqObjectPool<CqMovingMicroPolygonKey>	m_thePool;
 }
 ;
 
@@ -688,6 +729,21 @@ public:
             delete( ( *ikey ) );
 	}
 
+    /** Overridden operator new to avoid the pool allocator from CqMicroPolygon.
+     */
+    void* operator new( size_t size )
+    {
+        return( malloc(size) );
+    }
+
+    /** Overridden operator delete to allocate micropolys from a pool.
+     */
+    void operator delete( void* p )
+    {
+        free( p );
+    }
+
+
 public:
     void	AppendKey( const CqVector3D& vA, const CqVector3D& vB, const CqVector3D& vC, const CqVector3D& vD, TqFloat time );
     void	DeleteVariables( TqBool all )
@@ -695,8 +751,8 @@ public:
 
     // Overrides from CqMicroPolygon
 	virtual void CalculateTotalBound();
-	virtual	CqBound	GetTotalBound() { return m_Bound; }
-    virtual const CqBound	GetTotalBound() const
+	virtual	CqBound&	GetTotalBound() { return m_Bound; }
+    virtual const CqBound&	GetTotalBound() const
     {
         return ( m_Bound );
     }
