@@ -39,9 +39,13 @@
 
 #include	"ri.h"
 
+#include	"MultiTimer.h"
+
+
 START_NAMESPACE( Aqsis )
 
-DEFINE_STATIC_MEMORYPOOL( CqMovingMicroPolygonKeyPoints, 512 );
+CqObjectPool<CqMovingMicroPolygonKeyPoints>	CqMovingMicroPolygonKeyPoints::m_thePool;
+CqObjectPool<CqMicroPolygonPoints>	CqMicroPolygonPoints::m_thePool;
 
 
 #define NBR_SEGMENTS 6
@@ -455,13 +459,13 @@ void CqMicroPolyGridPoints::Split( CqImageBuffer* pImage, long xmin, long xmax,
 
     const CqParameterTypedConstant<TqFloat, type_float, TqFloat>* pConstantWidthParam = pPoints->constantwidth( );
 
-    QGetRenderContext() ->Stats().MakeProject().Start();
+    TIMER_START("Project points")
     // Transform the whole grid to hybrid camera/raster space
 
     CqVector3D* pP;
     pVar(EnvVars_P) ->GetPointPtr( pP );
 
-    QGetRenderContext() ->Stats().MakeProject().Stop();
+    TIMER_STOP("Project points")
 
     TqInt iu;
     TqInt iTime, tTime = pSurface()->pTransform()->cTimes();
@@ -798,7 +802,7 @@ void CqMicroPolygonMotionPoints::CalculateTotalBound()
  * \param fForce Boolean flag to force the recalculation of the cached bound.
  */
 
-CqBound CqMicroPolygonMotionPoints::GetTotalBound( )
+CqBound& CqMicroPolygonMotionPoints::GetTotalBound( )
 {
     return ( m_Bound );
 }
