@@ -326,13 +326,28 @@ public:
     CqMicroPolygonPoints() : CqMicroPolygon()	{}
     virtual	~CqMicroPolygonPoints()	{}
 
+	/** Overridden operator new to allocate micropolys from a pool.
+     */
+    void* operator new( size_t size )
+    {
+        return( m_thePool.alloc() );
+    }
+
+    /** Overridden operator delete to allocate micropolys from a pool.
+     */
+    void operator delete( void* p )
+    {
+        m_thePool.free( reinterpret_cast<CqMicroPolygonPoints*>(p) );
+    }
+
 public:
     void Initialise( TqFloat radius )
     {
         m_radius = radius;
     }
-    virtual	CqBound			GetTotalBound( )
+    virtual	CqBound&			GetTotalBound( )
     {
+		static CqBound b;
         CqVector3D Pmin, Pmax;
         pGrid()->pVar(EnvVars_P)->GetPoint(Pmin, m_Index);
         Pmax = Pmin;
@@ -340,7 +355,9 @@ public:
         Pmin.y( Pmin.y() - m_radius );
         Pmax.x( Pmax.x() + m_radius );
         Pmax.y( Pmax.y() + m_radius );
-        return( CqBound( Pmin, Pmax ) );
+		b.vecMin() = Pmin;
+		b.vecMax() = Pmax;
+        return( b );
     }
     virtual	TqBool	Sample( const SqSampleData& sample, TqFloat& D, TqFloat time, TqBool UsingDof = TqFalse );
 
@@ -349,6 +366,8 @@ private:
     CqMicroPolygonPoints( const CqMicroPolygonPoints& From )	{}
 
     TqFloat	m_radius;
+
+	static	CqObjectPool<CqMicroPolygonPoints>	m_thePool;
 }
 ;
 
@@ -357,7 +376,7 @@ private:
  * Base lass for static micropolygons. Stores point information about the geometry of the micropoly.
  */
 
-class CqMovingMicroPolygonKeyPoints : public CqPoolable<CqMovingMicroPolygonKeyPoints, 512>
+class CqMovingMicroPolygonKeyPoints
 {
 public:
     CqMovingMicroPolygonKeyPoints()
@@ -369,6 +388,20 @@ public:
     ~CqMovingMicroPolygonKeyPoints()
     {}
 
+	/** Overridden operator new to allocate micropolys from a pool.
+     */
+    void* operator new( size_t size )
+    {
+        return( m_thePool.alloc() );
+    }
+
+    /** Overridden operator delete to allocate micropolys from a pool.
+     */
+    void operator delete( void* p )
+    {
+        m_thePool.free( reinterpret_cast<CqMovingMicroPolygonKeyPoints*>(p) );
+    }
+
 public:
     TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const
     {
@@ -399,6 +432,8 @@ public:
 
     CqVector3D	m_Point0;
     TqFloat		m_radius;
+
+	static	CqObjectPool<CqMovingMicroPolygonKeyPoints>	m_thePool;
 }
 ;
 
@@ -427,8 +462,8 @@ void	DeleteVariables( TqBool all )	{}
     // Overrides from CqMicroPolygon
     virtual TqBool	fContains( const CqVector2D& vecP, TqFloat& Depth, TqFloat time ) const;
 	virtual void CalculateTotalBound();
-    virtual	CqBound			GetTotalBound( /*TqBool fForce = TqFalse */);
-    virtual const CqBound	GetTotalBound() const
+    virtual	CqBound&		GetTotalBound( /*TqBool fForce = TqFalse */);
+    virtual const CqBound&	GetTotalBound() const
     {
         return ( m_Bound );
     }
