@@ -170,10 +170,6 @@ CqRenderer::~CqRenderer()
 		delete m_pRaytracer;
 		m_pRaytracer = 0;	// MGC: or better NULL?
 	}
-	// Clear up the MicroPolygon memory pool.
-	CqMicroPolygon::Flush();
-	CqMovingMicroPolygonKeyPoints::Flush();
-	CqLath::Flush();
 
 	// Clear the ObjectInstance buffer
 	std::vector<CqObjectInstance*>::iterator i;
@@ -676,6 +672,8 @@ void CqRenderer::Initialise()
     m_OutputDataEntries.clear();
     m_OutputDataOffset = 9;		// Cs, Os, depth, coverage, a
     m_OutputDataTotalSize = 9;	// Cs, Os, depth, coverage, a
+
+	m_clippingVolume.clear();
 }
 
 
@@ -1079,7 +1077,14 @@ boost::shared_ptr<IqShader> CqRenderer::getDefaultSurfaceShader()
         boost::shared_ptr<IqShader> pMapCheck = 
                 CreateShader( "_def_", Type_Surface );
         if (pMapCheck)
+	{
+		// we must initialize the shader here.  non-default
+		//  shaders are initialized in RiSurfaceV()
+		CqShaderVM* pCreated = static_cast<CqShaderVM*>( pMapCheck.get() );
+		pCreated->PrepareDefArgs();
+		
                 return pMapCheck;
+	}
 
         // insert the default surface template into the map
         boost::shared_ptr<IqShader> pRet( new CqShaderVM() );
