@@ -24,6 +24,7 @@
 */
 
 #include	"aqsis.h"
+#include	"MultiTimer.h"
 
 #include	<math.h>
 #include	<iostream>
@@ -174,7 +175,7 @@ void CqShadowMap::LoadZFile()
             // Check validity of shadow map.
             if ( strncmp( strHeader, origHeader, headerLength ) != 0 )
             {
-                std::cerr << error << "Invalid shadow map format \"" << m_strName.c_str() << "\"" << std::endl;
+                std::cerr << error << "Invalid shadow map format \"" << m_strName.c_str() << "\"" << " : \"" << strHeader << "\"" << std::endl;
                 return ;
             }
 
@@ -203,6 +204,7 @@ void CqShadowMap::LoadZFile()
             // Set the matrixes to general, not Identity as default.
             matWorldToCamera().SetfIdentity( TqFalse );
             matWorldToScreen().SetfIdentity( TqFalse );
+	    m_Directory = 0;
         }
         else
         {
@@ -330,7 +332,7 @@ void	CqShadowMap::SampleMap( CqVector3D& R1, CqVector3D& R2, CqVector3D& R3, CqV
     // Check the memory and make sure we don't abuse it
     CriticalMeasure();
 
-    QGetRenderContext() ->Stats().TextureMapTimer().Start();
+	TIME_SCOPE("Texture Mapping")
 
     // If no map defined, not in shadow.
     val.resize( 1 );
@@ -371,10 +373,14 @@ void	CqShadowMap::SampleMap( CqVector3D& R1, CqVector3D& R2, CqVector3D& R3, CqV
     TqFloat z = vecR1l.z();
 
     vecR1m = matCameraToMap * R1;
-    vecR2m = matCameraToMap * R2;
-    vecR3m = matCameraToMap * R3;
-    vecR4m = matCameraToMap * R4;
-
+    if ((R1 == R2) && (R2 == R3) && (R3 == R4))
+    {
+        vecR2m = vecR3m = vecR4m = vecR1m;
+    } else {
+        vecR2m = matCameraToMap * R2;
+        vecR3m = matCameraToMap * R3;
+        vecR4m = matCameraToMap * R4;
+    }
 
     TqFloat sbo2 = m_sblur * (m_XRes - 1) * 0.5;
     TqFloat tbo2 = m_tblur * (m_YRes - 1) * 0.5;
@@ -426,7 +432,11 @@ void	CqShadowMap::SampleMap( CqVector3D& R1, CqVector3D& R2, CqVector3D& R3, CqV
     TqInt nt, ns;
     if ( m_samples > 0 )
     {
-        nt = ns = static_cast<TqInt>( CEIL( sqrt( m_samples ) ) );
+		if (m_samples == 16.0) nt = ns = 4;
+		else if (m_samples == 4.0) nt = ns = 2;
+		else if (m_samples == 256.0) nt = ns = 16;
+		else if (m_samples == 64.0) nt = ns = 8;
+		else nt = ns = static_cast<TqInt>( CEIL( sqrt( m_samples ) ) );
     }
     else
     {
@@ -508,8 +518,6 @@ void	CqShadowMap::SampleMap( CqVector3D& R1, CqVector3D& R2, CqVector3D& R3, CqV
     }
 
     val[ 0 ] = ( static_cast<TqFloat>( inshadow ) / ( ns * nt ) );
-
-    QGetRenderContext() ->Stats().TextureMapTimer().Stop();
 }
 
 
@@ -519,6 +527,8 @@ void	CqShadowMap::SampleMap( CqVector3D& R1, CqVector3D& R2, CqVector3D& R3, CqV
 
 void CqShadowMap::SaveShadowMap( const CqString& strShadowName, TqBool append )
 {
+    TqChar version[ 80 ];
+
     const char* mode = (append)? "a" : "w";
 
     // Save the shadowmap to a binary file.
@@ -541,6 +551,8 @@ void CqShadowMap::SaveShadowMap( const CqString& strShadowName, TqBool append )
                     matWToS[ ( r * 4 ) + c ] = matWorldToScreen()[ r ][ c ];
                 }
             }
+            sprintf( version, "%s %s", STRNAME, VERSION_STR );
+            TIFFSetField( pshadow, TIFFTAG_SOFTWARE, ( char* ) version );
             TIFFSetField( pshadow, TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA, matWToC );
             TIFFSetField( pshadow, TIFFTAG_PIXAR_MATRIX_WORLDTOSCREEN, matWToS );
             TIFFSetField( pshadow, TIFFTAG_PIXAR_TEXTUREFORMAT, SHADOWMAP_HEADER );
